/* $Id: html_utils.c,v 1.9 2020/02/07 03:51:46 rlr Exp $ */

/*
 * Copyright (c) 2018, 2020 Ross L Richardson <rueda@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>

#include "html_utils.h"
#include "uc_utils.h"
#include "utf8_utils.h"


/* UTF-8 encoding of UTF8_REPLACEMENT_CP... */
#define REPL      "\357\277\275"
#define REPL_LEN  3


/* Paragraph/line-break states. */
enum tth_state {
	TTH_START,
	TTH_IN_P,
	TTH_BR,
	TTH_EOP
};


/* HTML named character reference and UTF-8 encoded equivalent. */
struct	hncr_info {
	const char	*name;
	const char	*utf8;
};


static const struct hncr_info	 html_named_char_refs[] = {
	{ "&AElig;", "\303\206" },
	{ "&AMP;", "\46" },	/* & */
	{ "&Aacute;", "\303\201" },
	{ "&Abreve;", "\304\202" },
	{ "&Acirc;", "\303\202" },
	{ "&Acy;", "\320\220" },
	{ "&Afr;", "\355\240\265\355\264\204" },
	{ "&Agrave;", "\303\200" },
	{ "&Alpha;", "\316\221" },
	{ "&Amacr;", "\304\200" },
	{ "&And;", "\342\251\223" },
	{ "&Aogon;", "\304\204" },
	{ "&Aopf;", "\355\240\265\355\264\270" },
	{ "&ApplyFunction;", "\342\201\241" },
	{ "&Aring;", "\303\205" },
	{ "&Ascr;", "\355\240\265\355\262\234" },
	{ "&Assign;", "\342\211\224" },
	{ "&Atilde;", "\303\203" },
	{ "&Auml;", "\303\204" },
	{ "&Backslash;", "\342\210\226" },
	{ "&Barv;", "\342\253\247" },
	{ "&Barwed;", "\342\214\206" },
	{ "&Bcy;", "\320\221" },
	{ "&Because;", "\342\210\265" },
	{ "&Bernoullis;", "\342\204\254" },
	{ "&Beta;", "\316\222" },
	{ "&Bfr;", "\355\240\265\355\264\205" },
	{ "&Bopf;", "\355\240\265\355\264\271" },
	{ "&Breve;", "\313\230" },
	{ "&Bscr;", "\342\204\254" },
	{ "&Bumpeq;", "\342\211\216" },
	{ "&CHcy;", "\320\247" },
	{ "&COPY;", "\302\251" },
	{ "&Cacute;", "\304\206" },
	{ "&Cap;", "\342\213\222" },
	{ "&CapitalDifferentialD;", "\342\205\205" },
	{ "&Cayleys;", "\342\204\255" },
	{ "&Ccaron;", "\304\214" },
	{ "&Ccedil;", "\303\207" },
	{ "&Ccirc;", "\304\210" },
	{ "&Cconint;", "\342\210\260" },
	{ "&Cdot;", "\304\212" },
	{ "&Cedilla;", "\302\270" },
	{ "&CenterDot;", "\302\267" },
	{ "&Cfr;", "\342\204\255" },
	{ "&Chi;", "\316\247" },
	{ "&CircleDot;", "\342\212\231" },
	{ "&CircleMinus;", "\342\212\226" },
	{ "&CirclePlus;", "\342\212\225" },
	{ "&CircleTimes;", "\342\212\227" },
	{ "&ClockwiseContourIntegral;", "\342\210\262" },
	{ "&CloseCurlyDoubleQuote;", "\342\200\235" },
	{ "&CloseCurlyQuote;", "\342\200\231" },
	{ "&Colon;", "\342\210\267" },
	{ "&Colone;", "\342\251\264" },
	{ "&Congruent;", "\342\211\241" },
	{ "&Conint;", "\342\210\257" },
	{ "&ContourIntegral;", "\342\210\256" },
	{ "&Copf;", "\342\204\202" },
	{ "&Coproduct;", "\342\210\220" },
	{ "&CounterClockwiseContourIntegral;", "\342\210\263" },
	{ "&Cross;", "\342\250\257" },
	{ "&Cscr;", "\355\240\265\355\262\236" },
	{ "&Cup;", "\342\213\223" },
	{ "&CupCap;", "\342\211\215" },
	{ "&DD;", "\342\205\205" },
	{ "&DDotrahd;", "\342\244\221" },
	{ "&DJcy;", "\320\202" },
	{ "&DScy;", "\320\205" },
	{ "&DZcy;", "\320\217" },
	{ "&Dagger;", "\342\200\241" },
	{ "&Darr;", "\342\206\241" },
	{ "&Dashv;", "\342\253\244" },
	{ "&Dcaron;", "\304\216" },
	{ "&Dcy;", "\320\224" },
	{ "&Del;", "\342\210\207" },
	{ "&Delta;", "\316\224" },
	{ "&Dfr;", "\355\240\265\355\264\207" },
	{ "&DiacriticalAcute;", "\302\264" },
	{ "&DiacriticalDot;", "\313\231" },
	{ "&DiacriticalDoubleAcute;", "\313\235" },
	{ "&DiacriticalGrave;", "\140" },	/* ` */
	{ "&DiacriticalTilde;", "\313\234" },
	{ "&Diamond;", "\342\213\204" },
	{ "&DifferentialD;", "\342\205\206" },
	{ "&Dopf;", "\355\240\265\355\264\273" },
	{ "&Dot;", "\302\250" },
	{ "&DotDot;", "\342\203\234" },
	{ "&DotEqual;", "\342\211\220" },
	{ "&DoubleContourIntegral;", "\342\210\257" },
	{ "&DoubleDot;", "\302\250" },
	{ "&DoubleDownArrow;", "\342\207\223" },
	{ "&DoubleLeftArrow;", "\342\207\220" },
	{ "&DoubleLeftRightArrow;", "\342\207\224" },
	{ "&DoubleLeftTee;", "\342\253\244" },
	{ "&DoubleLongLeftArrow;", "\342\237\270" },
	{ "&DoubleLongLeftRightArrow;", "\342\237\272" },
	{ "&DoubleLongRightArrow;", "\342\237\271" },
	{ "&DoubleRightArrow;", "\342\207\222" },
	{ "&DoubleRightTee;", "\342\212\250" },
	{ "&DoubleUpArrow;", "\342\207\221" },
	{ "&DoubleUpDownArrow;", "\342\207\225" },
	{ "&DoubleVerticalBar;", "\342\210\245" },
	{ "&DownArrow;", "\342\206\223" },
	{ "&DownArrowBar;", "\342\244\223" },
	{ "&DownArrowUpArrow;", "\342\207\265" },
	{ "&DownBreve;", "\314\221" },
	{ "&DownLeftRightVector;", "\342\245\220" },
	{ "&DownLeftTeeVector;", "\342\245\236" },
	{ "&DownLeftVector;", "\342\206\275" },
	{ "&DownLeftVectorBar;", "\342\245\226" },
	{ "&DownRightTeeVector;", "\342\245\237" },
	{ "&DownRightVector;", "\342\207\201" },
	{ "&DownRightVectorBar;", "\342\245\227" },
	{ "&DownTee;", "\342\212\244" },
	{ "&DownTeeArrow;", "\342\206\247" },
	{ "&Downarrow;", "\342\207\223" },
	{ "&Dscr;", "\355\240\265\355\262\237" },
	{ "&Dstrok;", "\304\220" },
	{ "&ENG;", "\305\212" },
	{ "&ETH;", "\303\220" },
	{ "&Eacute;", "\303\211" },
	{ "&Ecaron;", "\304\232" },
	{ "&Ecirc;", "\303\212" },
	{ "&Ecy;", "\320\255" },
	{ "&Edot;", "\304\226" },
	{ "&Efr;", "\355\240\265\355\264\210" },
	{ "&Egrave;", "\303\210" },
	{ "&Element;", "\342\210\210" },
	{ "&Emacr;", "\304\222" },
	{ "&EmptySmallSquare;", "\342\227\273" },
	{ "&EmptyVerySmallSquare;", "\342\226\253" },
	{ "&Eogon;", "\304\230" },
	{ "&Eopf;", "\355\240\265\355\264\274" },
	{ "&Epsilon;", "\316\225" },
	{ "&Equal;", "\342\251\265" },
	{ "&EqualTilde;", "\342\211\202" },
	{ "&Equilibrium;", "\342\207\214" },
	{ "&Escr;", "\342\204\260" },
	{ "&Esim;", "\342\251\263" },
	{ "&Eta;", "\316\227" },
	{ "&Euml;", "\303\213" },
	{ "&Exists;", "\342\210\203" },
	{ "&ExponentialE;", "\342\205\207" },
	{ "&Fcy;", "\320\244" },
	{ "&Ffr;", "\355\240\265\355\264\211" },
	{ "&FilledSmallSquare;", "\342\227\274" },
	{ "&FilledVerySmallSquare;", "\342\226\252" },
	{ "&Fopf;", "\355\240\265\355\264\275" },
	{ "&ForAll;", "\342\210\200" },
	{ "&Fouriertrf;", "\342\204\261" },
	{ "&Fscr;", "\342\204\261" },
	{ "&GJcy;", "\320\203" },
	{ "&GT;", "\76" },	/* > */
	{ "&Gamma;", "\316\223" },
	{ "&Gammad;", "\317\234" },
	{ "&Gbreve;", "\304\236" },
	{ "&Gcedil;", "\304\242" },
	{ "&Gcirc;", "\304\234" },
	{ "&Gcy;", "\320\223" },
	{ "&Gdot;", "\304\240" },
	{ "&Gfr;", "\355\240\265\355\264\212" },
	{ "&Gg;", "\342\213\231" },
	{ "&Gopf;", "\355\240\265\355\264\276" },
	{ "&GreaterEqual;", "\342\211\245" },
	{ "&GreaterEqualLess;", "\342\213\233" },
	{ "&GreaterFullEqual;", "\342\211\247" },
	{ "&GreaterGreater;", "\342\252\242" },
	{ "&GreaterLess;", "\342\211\267" },
	{ "&GreaterSlantEqual;", "\342\251\276" },
	{ "&GreaterTilde;", "\342\211\263" },
	{ "&Gscr;", "\355\240\265\355\262\242" },
	{ "&Gt;", "\342\211\253" },
	{ "&HARDcy;", "\320\252" },
	{ "&Hacek;", "\313\207" },
	{ "&Hat;", "\136" },	/* ^ */
	{ "&Hcirc;", "\304\244" },
	{ "&Hfr;", "\342\204\214" },
	{ "&HilbertSpace;", "\342\204\213" },
	{ "&Hopf;", "\342\204\215" },
	{ "&HorizontalLine;", "\342\224\200" },
	{ "&Hscr;", "\342\204\213" },
	{ "&Hstrok;", "\304\246" },
	{ "&HumpDownHump;", "\342\211\216" },
	{ "&HumpEqual;", "\342\211\217" },
	{ "&IEcy;", "\320\225" },
	{ "&IJlig;", "\304\262" },
	{ "&IOcy;", "\320\201" },
	{ "&Iacute;", "\303\215" },
	{ "&Icirc;", "\303\216" },
	{ "&Icy;", "\320\230" },
	{ "&Idot;", "\304\260" },
	{ "&Ifr;", "\342\204\221" },
	{ "&Igrave;", "\303\214" },
	{ "&Im;", "\342\204\221" },
	{ "&Imacr;", "\304\252" },
	{ "&ImaginaryI;", "\342\205\210" },
	{ "&Implies;", "\342\207\222" },
	{ "&Int;", "\342\210\254" },
	{ "&Integral;", "\342\210\253" },
	{ "&Intersection;", "\342\213\202" },
	{ "&InvisibleComma;", "\342\201\243" },
	{ "&InvisibleTimes;", "\342\201\242" },
	{ "&Iogon;", "\304\256" },
	{ "&Iopf;", "\355\240\265\355\265\200" },
	{ "&Iota;", "\316\231" },
	{ "&Iscr;", "\342\204\220" },
	{ "&Itilde;", "\304\250" },
	{ "&Iukcy;", "\320\206" },
	{ "&Iuml;", "\303\217" },
	{ "&Jcirc;", "\304\264" },
	{ "&Jcy;", "\320\231" },
	{ "&Jfr;", "\355\240\265\355\264\215" },
	{ "&Jopf;", "\355\240\265\355\265\201" },
	{ "&Jscr;", "\355\240\265\355\262\245" },
	{ "&Jsercy;", "\320\210" },
	{ "&Jukcy;", "\320\204" },
	{ "&KHcy;", "\320\245" },
	{ "&KJcy;", "\320\214" },
	{ "&Kappa;", "\316\232" },
	{ "&Kcedil;", "\304\266" },
	{ "&Kcy;", "\320\232" },
	{ "&Kfr;", "\355\240\265\355\264\216" },
	{ "&Kopf;", "\355\240\265\355\265\202" },
	{ "&Kscr;", "\355\240\265\355\262\246" },
	{ "&LJcy;", "\320\211" },
	{ "&LT;", "\74" },	/* < */
	{ "&Lacute;", "\304\271" },
	{ "&Lambda;", "\316\233" },
	{ "&Lang;", "\342\237\252" },
	{ "&Laplacetrf;", "\342\204\222" },
	{ "&Larr;", "\342\206\236" },
	{ "&Lcaron;", "\304\275" },
	{ "&Lcedil;", "\304\273" },
	{ "&Lcy;", "\320\233" },
	{ "&LeftAngleBracket;", "\342\237\250" },
	{ "&LeftArrow;", "\342\206\220" },
	{ "&LeftArrowBar;", "\342\207\244" },
	{ "&LeftArrowRightArrow;", "\342\207\206" },
	{ "&LeftCeiling;", "\342\214\210" },
	{ "&LeftDoubleBracket;", "\342\237\246" },
	{ "&LeftDownTeeVector;", "\342\245\241" },
	{ "&LeftDownVector;", "\342\207\203" },
	{ "&LeftDownVectorBar;", "\342\245\231" },
	{ "&LeftFloor;", "\342\214\212" },
	{ "&LeftRightArrow;", "\342\206\224" },
	{ "&LeftRightVector;", "\342\245\216" },
	{ "&LeftTee;", "\342\212\243" },
	{ "&LeftTeeArrow;", "\342\206\244" },
	{ "&LeftTeeVector;", "\342\245\232" },
	{ "&LeftTriangle;", "\342\212\262" },
	{ "&LeftTriangleBar;", "\342\247\217" },
	{ "&LeftTriangleEqual;", "\342\212\264" },
	{ "&LeftUpDownVector;", "\342\245\221" },
	{ "&LeftUpTeeVector;", "\342\245\240" },
	{ "&LeftUpVector;", "\342\206\277" },
	{ "&LeftUpVectorBar;", "\342\245\230" },
	{ "&LeftVector;", "\342\206\274" },
	{ "&LeftVectorBar;", "\342\245\222" },
	{ "&Leftarrow;", "\342\207\220" },
	{ "&Leftrightarrow;", "\342\207\224" },
	{ "&LessEqualGreater;", "\342\213\232" },
	{ "&LessFullEqual;", "\342\211\246" },
	{ "&LessGreater;", "\342\211\266" },
	{ "&LessLess;", "\342\252\241" },
	{ "&LessSlantEqual;", "\342\251\275" },
	{ "&LessTilde;", "\342\211\262" },
	{ "&Lfr;", "\355\240\265\355\264\217" },
	{ "&Ll;", "\342\213\230" },
	{ "&Lleftarrow;", "\342\207\232" },
	{ "&Lmidot;", "\304\277" },
	{ "&LongLeftArrow;", "\342\237\265" },
	{ "&LongLeftRightArrow;", "\342\237\267" },
	{ "&LongRightArrow;", "\342\237\266" },
	{ "&Longleftarrow;", "\342\237\270" },
	{ "&Longleftrightarrow;", "\342\237\272" },
	{ "&Longrightarrow;", "\342\237\271" },
	{ "&Lopf;", "\355\240\265\355\265\203" },
	{ "&LowerLeftArrow;", "\342\206\231" },
	{ "&LowerRightArrow;", "\342\206\230" },
	{ "&Lscr;", "\342\204\222" },
	{ "&Lsh;", "\342\206\260" },
	{ "&Lstrok;", "\305\201" },
	{ "&Lt;", "\342\211\252" },
	{ "&Map;", "\342\244\205" },
	{ "&Mcy;", "\320\234" },
	{ "&MediumSpace;", "\342\201\237" },
	{ "&Mellintrf;", "\342\204\263" },
	{ "&Mfr;", "\355\240\265\355\264\220" },
	{ "&MinusPlus;", "\342\210\223" },
	{ "&Mopf;", "\355\240\265\355\265\204" },
	{ "&Mscr;", "\342\204\263" },
	{ "&Mu;", "\316\234" },
	{ "&NJcy;", "\320\212" },
	{ "&Nacute;", "\305\203" },
	{ "&Ncaron;", "\305\207" },
	{ "&Ncedil;", "\305\205" },
	{ "&Ncy;", "\320\235" },
	{ "&NegativeMediumSpace;", "\342\200\213" },
	{ "&NegativeThickSpace;", "\342\200\213" },
	{ "&NegativeThinSpace;", "\342\200\213" },
	{ "&NegativeVeryThinSpace;", "\342\200\213" },
	{ "&NestedGreaterGreater;", "\342\211\253" },
	{ "&NestedLessLess;", "\342\211\252" },
	{ "&NewLine;", "\12" },	/* \n */
	{ "&Nfr;", "\355\240\265\355\264\221" },
	{ "&NoBreak;", "\342\201\240" },
	{ "&NonBreakingSpace;", "\302\240" },
	{ "&Nopf;", "\342\204\225" },
	{ "&Not;", "\342\253\254" },
	{ "&NotCongruent;", "\342\211\242" },
	{ "&NotCupCap;", "\342\211\255" },
	{ "&NotDoubleVerticalBar;", "\342\210\246" },
	{ "&NotElement;", "\342\210\211" },
	{ "&NotEqual;", "\342\211\240" },
	{ "&NotEqualTilde;", "\342\211\202\314\270" },
	{ "&NotExists;", "\342\210\204" },
	{ "&NotGreater;", "\342\211\257" },
	{ "&NotGreaterEqual;", "\342\211\261" },
	{ "&NotGreaterFullEqual;", "\342\211\247\314\270" },
	{ "&NotGreaterGreater;", "\342\211\253\314\270" },
	{ "&NotGreaterLess;", "\342\211\271" },
	{ "&NotGreaterSlantEqual;", "\342\251\276\314\270" },
	{ "&NotGreaterTilde;", "\342\211\265" },
	{ "&NotHumpDownHump;", "\342\211\216\314\270" },
	{ "&NotHumpEqual;", "\342\211\217\314\270" },
	{ "&NotLeftTriangle;", "\342\213\252" },
	{ "&NotLeftTriangleBar;", "\342\247\217\314\270" },
	{ "&NotLeftTriangleEqual;", "\342\213\254" },
	{ "&NotLess;", "\342\211\256" },
	{ "&NotLessEqual;", "\342\211\260" },
	{ "&NotLessGreater;", "\342\211\270" },
	{ "&NotLessLess;", "\342\211\252\314\270" },
	{ "&NotLessSlantEqual;", "\342\251\275\314\270" },
	{ "&NotLessTilde;", "\342\211\264" },
	{ "&NotNestedGreaterGreater;", "\342\252\242\314\270" },
	{ "&NotNestedLessLess;", "\342\252\241\314\270" },
	{ "&NotPrecedes;", "\342\212\200" },
	{ "&NotPrecedesEqual;", "\342\252\257\314\270" },
	{ "&NotPrecedesSlantEqual;", "\342\213\240" },
	{ "&NotReverseElement;", "\342\210\214" },
	{ "&NotRightTriangle;", "\342\213\253" },
	{ "&NotRightTriangleBar;", "\342\247\220\314\270" },
	{ "&NotRightTriangleEqual;", "\342\213\255" },
	{ "&NotSquareSubset;", "\342\212\217\314\270" },
	{ "&NotSquareSubsetEqual;", "\342\213\242" },
	{ "&NotSquareSuperset;", "\342\212\220\314\270" },
	{ "&NotSquareSupersetEqual;", "\342\213\243" },
	{ "&NotSubset;", "\342\212\202\342\203\222" },
	{ "&NotSubsetEqual;", "\342\212\210" },
	{ "&NotSucceeds;", "\342\212\201" },
	{ "&NotSucceedsEqual;", "\342\252\260\314\270" },
	{ "&NotSucceedsSlantEqual;", "\342\213\241" },
	{ "&NotSucceedsTilde;", "\342\211\277\314\270" },
	{ "&NotSuperset;", "\342\212\203\342\203\222" },
	{ "&NotSupersetEqual;", "\342\212\211" },
	{ "&NotTilde;", "\342\211\201" },
	{ "&NotTildeEqual;", "\342\211\204" },
	{ "&NotTildeFullEqual;", "\342\211\207" },
	{ "&NotTildeTilde;", "\342\211\211" },
	{ "&NotVerticalBar;", "\342\210\244" },
	{ "&Nscr;", "\355\240\265\355\262\251" },
	{ "&Ntilde;", "\303\221" },
	{ "&Nu;", "\316\235" },
	{ "&OElig;", "\305\222" },
	{ "&Oacute;", "\303\223" },
	{ "&Ocirc;", "\303\224" },
	{ "&Ocy;", "\320\236" },
	{ "&Odblac;", "\305\220" },
	{ "&Ofr;", "\355\240\265\355\264\222" },
	{ "&Ograve;", "\303\222" },
	{ "&Omacr;", "\305\214" },
	{ "&Omega;", "\316\251" },
	{ "&Omicron;", "\316\237" },
	{ "&Oopf;", "\355\240\265\355\265\206" },
	{ "&OpenCurlyDoubleQuote;", "\342\200\234" },
	{ "&OpenCurlyQuote;", "\342\200\230" },
	{ "&Or;", "\342\251\224" },
	{ "&Oscr;", "\355\240\265\355\262\252" },
	{ "&Oslash;", "\303\230" },
	{ "&Otilde;", "\303\225" },
	{ "&Otimes;", "\342\250\267" },
	{ "&Ouml;", "\303\226" },
	{ "&OverBar;", "\342\200\276" },
	{ "&OverBrace;", "\342\217\236" },
	{ "&OverBracket;", "\342\216\264" },
	{ "&OverParenthesis;", "\342\217\234" },
	{ "&PartialD;", "\342\210\202" },
	{ "&Pcy;", "\320\237" },
	{ "&Pfr;", "\355\240\265\355\264\223" },
	{ "&Phi;", "\316\246" },
	{ "&Pi;", "\316\240" },
	{ "&PlusMinus;", "\302\261" },
	{ "&Poincareplane;", "\342\204\214" },
	{ "&Popf;", "\342\204\231" },
	{ "&Pr;", "\342\252\273" },
	{ "&Precedes;", "\342\211\272" },
	{ "&PrecedesEqual;", "\342\252\257" },
	{ "&PrecedesSlantEqual;", "\342\211\274" },
	{ "&PrecedesTilde;", "\342\211\276" },
	{ "&Prime;", "\342\200\263" },
	{ "&Product;", "\342\210\217" },
	{ "&Proportion;", "\342\210\267" },
	{ "&Proportional;", "\342\210\235" },
	{ "&Pscr;", "\355\240\265\355\262\253" },
	{ "&Psi;", "\316\250" },
	{ "&QUOT;", "\42" },	/* " */
	{ "&Qfr;", "\355\240\265\355\264\224" },
	{ "&Qopf;", "\342\204\232" },
	{ "&Qscr;", "\355\240\265\355\262\254" },
	{ "&RBarr;", "\342\244\220" },
	{ "&REG;", "\302\256" },
	{ "&Racute;", "\305\224" },
	{ "&Rang;", "\342\237\253" },
	{ "&Rarr;", "\342\206\240" },
	{ "&Rarrtl;", "\342\244\226" },
	{ "&Rcaron;", "\305\230" },
	{ "&Rcedil;", "\305\226" },
	{ "&Rcy;", "\320\240" },
	{ "&Re;", "\342\204\234" },
	{ "&ReverseElement;", "\342\210\213" },
	{ "&ReverseEquilibrium;", "\342\207\213" },
	{ "&ReverseUpEquilibrium;", "\342\245\257" },
	{ "&Rfr;", "\342\204\234" },
	{ "&Rho;", "\316\241" },
	{ "&RightAngleBracket;", "\342\237\251" },
	{ "&RightArrow;", "\342\206\222" },
	{ "&RightArrowBar;", "\342\207\245" },
	{ "&RightArrowLeftArrow;", "\342\207\204" },
	{ "&RightCeiling;", "\342\214\211" },
	{ "&RightDoubleBracket;", "\342\237\247" },
	{ "&RightDownTeeVector;", "\342\245\235" },
	{ "&RightDownVector;", "\342\207\202" },
	{ "&RightDownVectorBar;", "\342\245\225" },
	{ "&RightFloor;", "\342\214\213" },
	{ "&RightTee;", "\342\212\242" },
	{ "&RightTeeArrow;", "\342\206\246" },
	{ "&RightTeeVector;", "\342\245\233" },
	{ "&RightTriangle;", "\342\212\263" },
	{ "&RightTriangleBar;", "\342\247\220" },
	{ "&RightTriangleEqual;", "\342\212\265" },
	{ "&RightUpDownVector;", "\342\245\217" },
	{ "&RightUpTeeVector;", "\342\245\234" },
	{ "&RightUpVector;", "\342\206\276" },
	{ "&RightUpVectorBar;", "\342\245\224" },
	{ "&RightVector;", "\342\207\200" },
	{ "&RightVectorBar;", "\342\245\223" },
	{ "&Rightarrow;", "\342\207\222" },
	{ "&Ropf;", "\342\204\235" },
	{ "&RoundImplies;", "\342\245\260" },
	{ "&Rrightarrow;", "\342\207\233" },
	{ "&Rscr;", "\342\204\233" },
	{ "&Rsh;", "\342\206\261" },
	{ "&RuleDelayed;", "\342\247\264" },
	{ "&SHCHcy;", "\320\251" },
	{ "&SHcy;", "\320\250" },
	{ "&SOFTcy;", "\320\254" },
	{ "&Sacute;", "\305\232" },
	{ "&Sc;", "\342\252\274" },
	{ "&Scaron;", "\305\240" },
	{ "&Scedil;", "\305\236" },
	{ "&Scirc;", "\305\234" },
	{ "&Scy;", "\320\241" },
	{ "&Sfr;", "\355\240\265\355\264\226" },
	{ "&ShortDownArrow;", "\342\206\223" },
	{ "&ShortLeftArrow;", "\342\206\220" },
	{ "&ShortRightArrow;", "\342\206\222" },
	{ "&ShortUpArrow;", "\342\206\221" },
	{ "&Sigma;", "\316\243" },
	{ "&SmallCircle;", "\342\210\230" },
	{ "&Sopf;", "\355\240\265\355\265\212" },
	{ "&Sqrt;", "\342\210\232" },
	{ "&Square;", "\342\226\241" },
	{ "&SquareIntersection;", "\342\212\223" },
	{ "&SquareSubset;", "\342\212\217" },
	{ "&SquareSubsetEqual;", "\342\212\221" },
	{ "&SquareSuperset;", "\342\212\220" },
	{ "&SquareSupersetEqual;", "\342\212\222" },
	{ "&SquareUnion;", "\342\212\224" },
	{ "&Sscr;", "\355\240\265\355\262\256" },
	{ "&Star;", "\342\213\206" },
	{ "&Sub;", "\342\213\220" },
	{ "&Subset;", "\342\213\220" },
	{ "&SubsetEqual;", "\342\212\206" },
	{ "&Succeeds;", "\342\211\273" },
	{ "&SucceedsEqual;", "\342\252\260" },
	{ "&SucceedsSlantEqual;", "\342\211\275" },
	{ "&SucceedsTilde;", "\342\211\277" },
	{ "&SuchThat;", "\342\210\213" },
	{ "&Sum;", "\342\210\221" },
	{ "&Sup;", "\342\213\221" },
	{ "&Superset;", "\342\212\203" },
	{ "&SupersetEqual;", "\342\212\207" },
	{ "&Supset;", "\342\213\221" },
	{ "&THORN;", "\303\236" },
	{ "&TRADE;", "\342\204\242" },
	{ "&TSHcy;", "\320\213" },
	{ "&TScy;", "\320\246" },
	{ "&Tab;", "\11" },	/* \t */
	{ "&Tau;", "\316\244" },
	{ "&Tcaron;", "\305\244" },
	{ "&Tcedil;", "\305\242" },
	{ "&Tcy;", "\320\242" },
	{ "&Tfr;", "\355\240\265\355\264\227" },
	{ "&Therefore;", "\342\210\264" },
	{ "&Theta;", "\316\230" },
	{ "&ThickSpace;", "\342\201\237\342\200\212" },
	{ "&ThinSpace;", "\342\200\211" },
	{ "&Tilde;", "\342\210\274" },
	{ "&TildeEqual;", "\342\211\203" },
	{ "&TildeFullEqual;", "\342\211\205" },
	{ "&TildeTilde;", "\342\211\210" },
	{ "&Topf;", "\355\240\265\355\265\213" },
	{ "&TripleDot;", "\342\203\233" },
	{ "&Tscr;", "\355\240\265\355\262\257" },
	{ "&Tstrok;", "\305\246" },
	{ "&Uacute;", "\303\232" },
	{ "&Uarr;", "\342\206\237" },
	{ "&Uarrocir;", "\342\245\211" },
	{ "&Ubrcy;", "\320\216" },
	{ "&Ubreve;", "\305\254" },
	{ "&Ucirc;", "\303\233" },
	{ "&Ucy;", "\320\243" },
	{ "&Udblac;", "\305\260" },
	{ "&Ufr;", "\355\240\265\355\264\230" },
	{ "&Ugrave;", "\303\231" },
	{ "&Umacr;", "\305\252" },
	{ "&UnderBar;", "\137" },	/* _ */
	{ "&UnderBrace;", "\342\217\237" },
	{ "&UnderBracket;", "\342\216\265" },
	{ "&UnderParenthesis;", "\342\217\235" },
	{ "&Union;", "\342\213\203" },
	{ "&UnionPlus;", "\342\212\216" },
	{ "&Uogon;", "\305\262" },
	{ "&Uopf;", "\355\240\265\355\265\214" },
	{ "&UpArrow;", "\342\206\221" },
	{ "&UpArrowBar;", "\342\244\222" },
	{ "&UpArrowDownArrow;", "\342\207\205" },
	{ "&UpDownArrow;", "\342\206\225" },
	{ "&UpEquilibrium;", "\342\245\256" },
	{ "&UpTee;", "\342\212\245" },
	{ "&UpTeeArrow;", "\342\206\245" },
	{ "&Uparrow;", "\342\207\221" },
	{ "&Updownarrow;", "\342\207\225" },
	{ "&UpperLeftArrow;", "\342\206\226" },
	{ "&UpperRightArrow;", "\342\206\227" },
	{ "&Upsi;", "\317\222" },
	{ "&Upsilon;", "\316\245" },
	{ "&Uring;", "\305\256" },
	{ "&Uscr;", "\355\240\265\355\262\260" },
	{ "&Utilde;", "\305\250" },
	{ "&Uuml;", "\303\234" },
	{ "&VDash;", "\342\212\253" },
	{ "&Vbar;", "\342\253\253" },
	{ "&Vcy;", "\320\222" },
	{ "&Vdash;", "\342\212\251" },
	{ "&Vdashl;", "\342\253\246" },
	{ "&Vee;", "\342\213\201" },
	{ "&Verbar;", "\342\200\226" },
	{ "&Vert;", "\342\200\226" },
	{ "&VerticalBar;", "\342\210\243" },
	{ "&VerticalLine;", "\174" },	/* | */
	{ "&VerticalSeparator;", "\342\235\230" },
	{ "&VerticalTilde;", "\342\211\200" },
	{ "&VeryThinSpace;", "\342\200\212" },
	{ "&Vfr;", "\355\240\265\355\264\231" },
	{ "&Vopf;", "\355\240\265\355\265\215" },
	{ "&Vscr;", "\355\240\265\355\262\261" },
	{ "&Vvdash;", "\342\212\252" },
	{ "&Wcirc;", "\305\264" },
	{ "&Wedge;", "\342\213\200" },
	{ "&Wfr;", "\355\240\265\355\264\232" },
	{ "&Wopf;", "\355\240\265\355\265\216" },
	{ "&Wscr;", "\355\240\265\355\262\262" },
	{ "&Xfr;", "\355\240\265\355\264\233" },
	{ "&Xi;", "\316\236" },
	{ "&Xopf;", "\355\240\265\355\265\217" },
	{ "&Xscr;", "\355\240\265\355\262\263" },
	{ "&YAcy;", "\320\257" },
	{ "&YIcy;", "\320\207" },
	{ "&YUcy;", "\320\256" },
	{ "&Yacute;", "\303\235" },
	{ "&Ycirc;", "\305\266" },
	{ "&Ycy;", "\320\253" },
	{ "&Yfr;", "\355\240\265\355\264\234" },
	{ "&Yopf;", "\355\240\265\355\265\220" },
	{ "&Yscr;", "\355\240\265\355\262\264" },
	{ "&Yuml;", "\305\270" },
	{ "&ZHcy;", "\320\226" },
	{ "&Zacute;", "\305\271" },
	{ "&Zcaron;", "\305\275" },
	{ "&Zcy;", "\320\227" },
	{ "&Zdot;", "\305\273" },
	{ "&ZeroWidthSpace;", "\342\200\213" },
	{ "&Zeta;", "\316\226" },
	{ "&Zfr;", "\342\204\250" },
	{ "&Zopf;", "\342\204\244" },
	{ "&Zscr;", "\355\240\265\355\262\265" },
	{ "&aacute;", "\303\241" },
	{ "&abreve;", "\304\203" },
	{ "&ac;", "\342\210\276" },
	{ "&acE;", "\342\210\276\314\263" },
	{ "&acd;", "\342\210\277" },
	{ "&acirc;", "\303\242" },
	{ "&acute;", "\302\264" },
	{ "&acy;", "\320\260" },
	{ "&aelig;", "\303\246" },
	{ "&af;", "\342\201\241" },
	{ "&afr;", "\355\240\265\355\264\236" },
	{ "&agrave;", "\303\240" },
	{ "&alefsym;", "\342\204\265" },
	{ "&aleph;", "\342\204\265" },
	{ "&alpha;", "\316\261" },
	{ "&amacr;", "\304\201" },
	{ "&amalg;", "\342\250\277" },
	{ "&amp;", "\46" },	/* & */
	{ "&and;", "\342\210\247" },
	{ "&andand;", "\342\251\225" },
	{ "&andd;", "\342\251\234" },
	{ "&andslope;", "\342\251\230" },
	{ "&andv;", "\342\251\232" },
	{ "&ang;", "\342\210\240" },
	{ "&ange;", "\342\246\244" },
	{ "&angle;", "\342\210\240" },
	{ "&angmsd;", "\342\210\241" },
	{ "&angmsdaa;", "\342\246\250" },
	{ "&angmsdab;", "\342\246\251" },
	{ "&angmsdac;", "\342\246\252" },
	{ "&angmsdad;", "\342\246\253" },
	{ "&angmsdae;", "\342\246\254" },
	{ "&angmsdaf;", "\342\246\255" },
	{ "&angmsdag;", "\342\246\256" },
	{ "&angmsdah;", "\342\246\257" },
	{ "&angrt;", "\342\210\237" },
	{ "&angrtvb;", "\342\212\276" },
	{ "&angrtvbd;", "\342\246\235" },
	{ "&angsph;", "\342\210\242" },
	{ "&angst;", "\303\205" },
	{ "&angzarr;", "\342\215\274" },
	{ "&aogon;", "\304\205" },
	{ "&aopf;", "\355\240\265\355\265\222" },
	{ "&ap;", "\342\211\210" },
	{ "&apE;", "\342\251\260" },
	{ "&apacir;", "\342\251\257" },
	{ "&ape;", "\342\211\212" },
	{ "&apid;", "\342\211\213" },
	{ "&apos;", "\47" },	/* ' */
	{ "&approx;", "\342\211\210" },
	{ "&approxeq;", "\342\211\212" },
	{ "&aring;", "\303\245" },
	{ "&ascr;", "\355\240\265\355\262\266" },
	{ "&ast;", "\52" },	/* * */
	{ "&asymp;", "\342\211\210" },
	{ "&asympeq;", "\342\211\215" },
	{ "&atilde;", "\303\243" },
	{ "&auml;", "\303\244" },
	{ "&awconint;", "\342\210\263" },
	{ "&awint;", "\342\250\221" },
	{ "&bNot;", "\342\253\255" },
	{ "&backcong;", "\342\211\214" },
	{ "&backepsilon;", "\317\266" },
	{ "&backprime;", "\342\200\265" },
	{ "&backsim;", "\342\210\275" },
	{ "&backsimeq;", "\342\213\215" },
	{ "&barvee;", "\342\212\275" },
	{ "&barwed;", "\342\214\205" },
	{ "&barwedge;", "\342\214\205" },
	{ "&bbrk;", "\342\216\265" },
	{ "&bbrktbrk;", "\342\216\266" },
	{ "&bcong;", "\342\211\214" },
	{ "&bcy;", "\320\261" },
	{ "&bdquo;", "\342\200\236" },
	{ "&becaus;", "\342\210\265" },
	{ "&because;", "\342\210\265" },
	{ "&bemptyv;", "\342\246\260" },
	{ "&bepsi;", "\317\266" },
	{ "&bernou;", "\342\204\254" },
	{ "&beta;", "\316\262" },
	{ "&beth;", "\342\204\266" },
	{ "&between;", "\342\211\254" },
	{ "&bfr;", "\355\240\265\355\264\237" },
	{ "&bigcap;", "\342\213\202" },
	{ "&bigcirc;", "\342\227\257" },
	{ "&bigcup;", "\342\213\203" },
	{ "&bigodot;", "\342\250\200" },
	{ "&bigoplus;", "\342\250\201" },
	{ "&bigotimes;", "\342\250\202" },
	{ "&bigsqcup;", "\342\250\206" },
	{ "&bigstar;", "\342\230\205" },
	{ "&bigtriangledown;", "\342\226\275" },
	{ "&bigtriangleup;", "\342\226\263" },
	{ "&biguplus;", "\342\250\204" },
	{ "&bigvee;", "\342\213\201" },
	{ "&bigwedge;", "\342\213\200" },
	{ "&bkarow;", "\342\244\215" },
	{ "&blacklozenge;", "\342\247\253" },
	{ "&blacksquare;", "\342\226\252" },
	{ "&blacktriangle;", "\342\226\264" },
	{ "&blacktriangledown;", "\342\226\276" },
	{ "&blacktriangleleft;", "\342\227\202" },
	{ "&blacktriangleright;", "\342\226\270" },
	{ "&blank;", "\342\220\243" },
	{ "&blk12;", "\342\226\222" },
	{ "&blk14;", "\342\226\221" },
	{ "&blk34;", "\342\226\223" },
	{ "&block;", "\342\226\210" },
	{ "&bne;", "\75\342\203\245" },
	{ "&bnequiv;", "\342\211\241\342\203\245" },
	{ "&bnot;", "\342\214\220" },
	{ "&bopf;", "\355\240\265\355\265\223" },
	{ "&bot;", "\342\212\245" },
	{ "&bottom;", "\342\212\245" },
	{ "&bowtie;", "\342\213\210" },
	{ "&boxDL;", "\342\225\227" },
	{ "&boxDR;", "\342\225\224" },
	{ "&boxDl;", "\342\225\226" },
	{ "&boxDr;", "\342\225\223" },
	{ "&boxH;", "\342\225\220" },
	{ "&boxHD;", "\342\225\246" },
	{ "&boxHU;", "\342\225\251" },
	{ "&boxHd;", "\342\225\244" },
	{ "&boxHu;", "\342\225\247" },
	{ "&boxUL;", "\342\225\235" },
	{ "&boxUR;", "\342\225\232" },
	{ "&boxUl;", "\342\225\234" },
	{ "&boxUr;", "\342\225\231" },
	{ "&boxV;", "\342\225\221" },
	{ "&boxVH;", "\342\225\254" },
	{ "&boxVL;", "\342\225\243" },
	{ "&boxVR;", "\342\225\240" },
	{ "&boxVh;", "\342\225\253" },
	{ "&boxVl;", "\342\225\242" },
	{ "&boxVr;", "\342\225\237" },
	{ "&boxbox;", "\342\247\211" },
	{ "&boxdL;", "\342\225\225" },
	{ "&boxdR;", "\342\225\222" },
	{ "&boxdl;", "\342\224\220" },
	{ "&boxdr;", "\342\224\214" },
	{ "&boxh;", "\342\224\200" },
	{ "&boxhD;", "\342\225\245" },
	{ "&boxhU;", "\342\225\250" },
	{ "&boxhd;", "\342\224\254" },
	{ "&boxhu;", "\342\224\264" },
	{ "&boxminus;", "\342\212\237" },
	{ "&boxplus;", "\342\212\236" },
	{ "&boxtimes;", "\342\212\240" },
	{ "&boxuL;", "\342\225\233" },
	{ "&boxuR;", "\342\225\230" },
	{ "&boxul;", "\342\224\230" },
	{ "&boxur;", "\342\224\224" },
	{ "&boxv;", "\342\224\202" },
	{ "&boxvH;", "\342\225\252" },
	{ "&boxvL;", "\342\225\241" },
	{ "&boxvR;", "\342\225\236" },
	{ "&boxvh;", "\342\224\274" },
	{ "&boxvl;", "\342\224\244" },
	{ "&boxvr;", "\342\224\234" },
	{ "&bprime;", "\342\200\265" },
	{ "&breve;", "\313\230" },
	{ "&brvbar;", "\302\246" },
	{ "&bscr;", "\355\240\265\355\262\267" },
	{ "&bsemi;", "\342\201\217" },
	{ "&bsim;", "\342\210\275" },
	{ "&bsime;", "\342\213\215" },
	{ "&bsol;", "\134" },	/* \\ */
	{ "&bsolb;", "\342\247\205" },
	{ "&bsolhsub;", "\342\237\210" },
	{ "&bull;", "\342\200\242" },
	{ "&bullet;", "\342\200\242" },
	{ "&bump;", "\342\211\216" },
	{ "&bumpE;", "\342\252\256" },
	{ "&bumpe;", "\342\211\217" },
	{ "&bumpeq;", "\342\211\217" },
	{ "&cacute;", "\304\207" },
	{ "&cap;", "\342\210\251" },
	{ "&capand;", "\342\251\204" },
	{ "&capbrcup;", "\342\251\211" },
	{ "&capcap;", "\342\251\213" },
	{ "&capcup;", "\342\251\207" },
	{ "&capdot;", "\342\251\200" },
	{ "&caps;", "\342\210\251\357\270\200" },
	{ "&caret;", "\342\201\201" },
	{ "&caron;", "\313\207" },
	{ "&ccaps;", "\342\251\215" },
	{ "&ccaron;", "\304\215" },
	{ "&ccedil;", "\303\247" },
	{ "&ccirc;", "\304\211" },
	{ "&ccups;", "\342\251\214" },
	{ "&ccupssm;", "\342\251\220" },
	{ "&cdot;", "\304\213" },
	{ "&cedil;", "\302\270" },
	{ "&cemptyv;", "\342\246\262" },
	{ "&cent;", "\302\242" },
	{ "&centerdot;", "\302\267" },
	{ "&cfr;", "\355\240\265\355\264\240" },
	{ "&chcy;", "\321\207" },
	{ "&check;", "\342\234\223" },
	{ "&checkmark;", "\342\234\223" },
	{ "&chi;", "\317\207" },
	{ "&cir;", "\342\227\213" },
	{ "&cirE;", "\342\247\203" },
	{ "&circ;", "\313\206" },
	{ "&circeq;", "\342\211\227" },
	{ "&circlearrowleft;", "\342\206\272" },
	{ "&circlearrowright;", "\342\206\273" },
	{ "&circledR;", "\302\256" },
	{ "&circledS;", "\342\223\210" },
	{ "&circledast;", "\342\212\233" },
	{ "&circledcirc;", "\342\212\232" },
	{ "&circleddash;", "\342\212\235" },
	{ "&cire;", "\342\211\227" },
	{ "&cirfnint;", "\342\250\220" },
	{ "&cirmid;", "\342\253\257" },
	{ "&cirscir;", "\342\247\202" },
	{ "&clubs;", "\342\231\243" },
	{ "&clubsuit;", "\342\231\243" },
	{ "&colon;", "\72" },	/* : */
	{ "&colone;", "\342\211\224" },
	{ "&coloneq;", "\342\211\224" },
	{ "&comma;", "\54" },	/* , */
	{ "&commat;", "\100" },	/* @ */
	{ "&comp;", "\342\210\201" },
	{ "&compfn;", "\342\210\230" },
	{ "&complement;", "\342\210\201" },
	{ "&complexes;", "\342\204\202" },
	{ "&cong;", "\342\211\205" },
	{ "&congdot;", "\342\251\255" },
	{ "&conint;", "\342\210\256" },
	{ "&copf;", "\355\240\265\355\265\224" },
	{ "&coprod;", "\342\210\220" },
	{ "&copy;", "\302\251" },
	{ "&copysr;", "\342\204\227" },
	{ "&crarr;", "\342\206\265" },
	{ "&cross;", "\342\234\227" },
	{ "&cscr;", "\355\240\265\355\262\270" },
	{ "&csub;", "\342\253\217" },
	{ "&csube;", "\342\253\221" },
	{ "&csup;", "\342\253\220" },
	{ "&csupe;", "\342\253\222" },
	{ "&ctdot;", "\342\213\257" },
	{ "&cudarrl;", "\342\244\270" },
	{ "&cudarrr;", "\342\244\265" },
	{ "&cuepr;", "\342\213\236" },
	{ "&cuesc;", "\342\213\237" },
	{ "&cularr;", "\342\206\266" },
	{ "&cularrp;", "\342\244\275" },
	{ "&cup;", "\342\210\252" },
	{ "&cupbrcap;", "\342\251\210" },
	{ "&cupcap;", "\342\251\206" },
	{ "&cupcup;", "\342\251\212" },
	{ "&cupdot;", "\342\212\215" },
	{ "&cupor;", "\342\251\205" },
	{ "&cups;", "\342\210\252\357\270\200" },
	{ "&curarr;", "\342\206\267" },
	{ "&curarrm;", "\342\244\274" },
	{ "&curlyeqprec;", "\342\213\236" },
	{ "&curlyeqsucc;", "\342\213\237" },
	{ "&curlyvee;", "\342\213\216" },
	{ "&curlywedge;", "\342\213\217" },
	{ "&curren;", "\302\244" },
	{ "&curvearrowleft;", "\342\206\266" },
	{ "&curvearrowright;", "\342\206\267" },
	{ "&cuvee;", "\342\213\216" },
	{ "&cuwed;", "\342\213\217" },
	{ "&cwconint;", "\342\210\262" },
	{ "&cwint;", "\342\210\261" },
	{ "&cylcty;", "\342\214\255" },
	{ "&dArr;", "\342\207\223" },
	{ "&dHar;", "\342\245\245" },
	{ "&dagger;", "\342\200\240" },
	{ "&daleth;", "\342\204\270" },
	{ "&darr;", "\342\206\223" },
	{ "&dash;", "\342\200\220" },
	{ "&dashv;", "\342\212\243" },
	{ "&dbkarow;", "\342\244\217" },
	{ "&dblac;", "\313\235" },
	{ "&dcaron;", "\304\217" },
	{ "&dcy;", "\320\264" },
	{ "&dd;", "\342\205\206" },
	{ "&ddagger;", "\342\200\241" },
	{ "&ddarr;", "\342\207\212" },
	{ "&ddotseq;", "\342\251\267" },
	{ "&deg;", "\302\260" },
	{ "&delta;", "\316\264" },
	{ "&demptyv;", "\342\246\261" },
	{ "&dfisht;", "\342\245\277" },
	{ "&dfr;", "\355\240\265\355\264\241" },
	{ "&dharl;", "\342\207\203" },
	{ "&dharr;", "\342\207\202" },
	{ "&diam;", "\342\213\204" },
	{ "&diamond;", "\342\213\204" },
	{ "&diamondsuit;", "\342\231\246" },
	{ "&diams;", "\342\231\246" },
	{ "&die;", "\302\250" },
	{ "&digamma;", "\317\235" },
	{ "&disin;", "\342\213\262" },
	{ "&div;", "\303\267" },
	{ "&divide;", "\303\267" },
	{ "&divideontimes;", "\342\213\207" },
	{ "&divonx;", "\342\213\207" },
	{ "&djcy;", "\321\222" },
	{ "&dlcorn;", "\342\214\236" },
	{ "&dlcrop;", "\342\214\215" },
	{ "&dollar;", "\44" },	/* $ */
	{ "&dopf;", "\355\240\265\355\265\225" },
	{ "&dot;", "\313\231" },
	{ "&doteq;", "\342\211\220" },
	{ "&doteqdot;", "\342\211\221" },
	{ "&dotminus;", "\342\210\270" },
	{ "&dotplus;", "\342\210\224" },
	{ "&dotsquare;", "\342\212\241" },
	{ "&doublebarwedge;", "\342\214\206" },
	{ "&downarrow;", "\342\206\223" },
	{ "&downdownarrows;", "\342\207\212" },
	{ "&downharpoonleft;", "\342\207\203" },
	{ "&downharpoonright;", "\342\207\202" },
	{ "&drbkarow;", "\342\244\220" },
	{ "&drcorn;", "\342\214\237" },
	{ "&drcrop;", "\342\214\214" },
	{ "&dscr;", "\355\240\265\355\262\271" },
	{ "&dscy;", "\321\225" },
	{ "&dsol;", "\342\247\266" },
	{ "&dstrok;", "\304\221" },
	{ "&dtdot;", "\342\213\261" },
	{ "&dtri;", "\342\226\277" },
	{ "&dtrif;", "\342\226\276" },
	{ "&duarr;", "\342\207\265" },
	{ "&duhar;", "\342\245\257" },
	{ "&dwangle;", "\342\246\246" },
	{ "&dzcy;", "\321\237" },
	{ "&dzigrarr;", "\342\237\277" },
	{ "&eDDot;", "\342\251\267" },
	{ "&eDot;", "\342\211\221" },
	{ "&eacute;", "\303\251" },
	{ "&easter;", "\342\251\256" },
	{ "&ecaron;", "\304\233" },
	{ "&ecir;", "\342\211\226" },
	{ "&ecirc;", "\303\252" },
	{ "&ecolon;", "\342\211\225" },
	{ "&ecy;", "\321\215" },
	{ "&edot;", "\304\227" },
	{ "&ee;", "\342\205\207" },
	{ "&efDot;", "\342\211\222" },
	{ "&efr;", "\355\240\265\355\264\242" },
	{ "&eg;", "\342\252\232" },
	{ "&egrave;", "\303\250" },
	{ "&egs;", "\342\252\226" },
	{ "&egsdot;", "\342\252\230" },
	{ "&el;", "\342\252\231" },
	{ "&elinters;", "\342\217\247" },
	{ "&ell;", "\342\204\223" },
	{ "&els;", "\342\252\225" },
	{ "&elsdot;", "\342\252\227" },
	{ "&emacr;", "\304\223" },
	{ "&empty;", "\342\210\205" },
	{ "&emptyset;", "\342\210\205" },
	{ "&emptyv;", "\342\210\205" },
	{ "&emsp13;", "\342\200\204" },
	{ "&emsp14;", "\342\200\205" },
	{ "&emsp;", "\342\200\203" },
	{ "&eng;", "\305\213" },
	{ "&ensp;", "\342\200\202" },
	{ "&eogon;", "\304\231" },
	{ "&eopf;", "\355\240\265\355\265\226" },
	{ "&epar;", "\342\213\225" },
	{ "&eparsl;", "\342\247\243" },
	{ "&eplus;", "\342\251\261" },
	{ "&epsi;", "\316\265" },
	{ "&epsilon;", "\316\265" },
	{ "&epsiv;", "\317\265" },
	{ "&eqcirc;", "\342\211\226" },
	{ "&eqcolon;", "\342\211\225" },
	{ "&eqsim;", "\342\211\202" },
	{ "&eqslantgtr;", "\342\252\226" },
	{ "&eqslantless;", "\342\252\225" },
	{ "&equals;", "\75" },	/* = */
	{ "&equest;", "\342\211\237" },
	{ "&equiv;", "\342\211\241" },
	{ "&equivDD;", "\342\251\270" },
	{ "&eqvparsl;", "\342\247\245" },
	{ "&erDot;", "\342\211\223" },
	{ "&erarr;", "\342\245\261" },
	{ "&escr;", "\342\204\257" },
	{ "&esdot;", "\342\211\220" },
	{ "&esim;", "\342\211\202" },
	{ "&eta;", "\316\267" },
	{ "&eth;", "\303\260" },
	{ "&euml;", "\303\253" },
	{ "&euro;", "\342\202\254" },
	{ "&excl;", "\41" },	/* ! */
	{ "&exist;", "\342\210\203" },
	{ "&expectation;", "\342\204\260" },
	{ "&exponentiale;", "\342\205\207" },
	{ "&fallingdotseq;", "\342\211\222" },
	{ "&fcy;", "\321\204" },
	{ "&female;", "\342\231\200" },
	{ "&ffilig;", "\357\254\203" },
	{ "&fflig;", "\357\254\200" },
	{ "&ffllig;", "\357\254\204" },
	{ "&ffr;", "\355\240\265\355\264\243" },
	{ "&filig;", "\357\254\201" },
	{ "&fjlig;", "\146\152" },
	{ "&flat;", "\342\231\255" },
	{ "&fllig;", "\357\254\202" },
	{ "&fltns;", "\342\226\261" },
	{ "&fnof;", "\306\222" },
	{ "&fopf;", "\355\240\265\355\265\227" },
	{ "&forall;", "\342\210\200" },
	{ "&fork;", "\342\213\224" },
	{ "&forkv;", "\342\253\231" },
	{ "&fpartint;", "\342\250\215" },
	{ "&frac12;", "\302\275" },
	{ "&frac13;", "\342\205\223" },
	{ "&frac14;", "\302\274" },
	{ "&frac15;", "\342\205\225" },
	{ "&frac16;", "\342\205\231" },
	{ "&frac18;", "\342\205\233" },
	{ "&frac23;", "\342\205\224" },
	{ "&frac25;", "\342\205\226" },
	{ "&frac34;", "\302\276" },
	{ "&frac35;", "\342\205\227" },
	{ "&frac38;", "\342\205\234" },
	{ "&frac45;", "\342\205\230" },
	{ "&frac56;", "\342\205\232" },
	{ "&frac58;", "\342\205\235" },
	{ "&frac78;", "\342\205\236" },
	{ "&frasl;", "\342\201\204" },
	{ "&frown;", "\342\214\242" },
	{ "&fscr;", "\355\240\265\355\262\273" },
	{ "&gE;", "\342\211\247" },
	{ "&gEl;", "\342\252\214" },
	{ "&gacute;", "\307\265" },
	{ "&gamma;", "\316\263" },
	{ "&gammad;", "\317\235" },
	{ "&gap;", "\342\252\206" },
	{ "&gbreve;", "\304\237" },
	{ "&gcirc;", "\304\235" },
	{ "&gcy;", "\320\263" },
	{ "&gdot;", "\304\241" },
	{ "&ge;", "\342\211\245" },
	{ "&gel;", "\342\213\233" },
	{ "&geq;", "\342\211\245" },
	{ "&geqq;", "\342\211\247" },
	{ "&geqslant;", "\342\251\276" },
	{ "&ges;", "\342\251\276" },
	{ "&gescc;", "\342\252\251" },
	{ "&gesdot;", "\342\252\200" },
	{ "&gesdoto;", "\342\252\202" },
	{ "&gesdotol;", "\342\252\204" },
	{ "&gesl;", "\342\213\233\357\270\200" },
	{ "&gesles;", "\342\252\224" },
	{ "&gfr;", "\355\240\265\355\264\244" },
	{ "&gg;", "\342\211\253" },
	{ "&ggg;", "\342\213\231" },
	{ "&gimel;", "\342\204\267" },
	{ "&gjcy;", "\321\223" },
	{ "&gl;", "\342\211\267" },
	{ "&glE;", "\342\252\222" },
	{ "&gla;", "\342\252\245" },
	{ "&glj;", "\342\252\244" },
	{ "&gnE;", "\342\211\251" },
	{ "&gnap;", "\342\252\212" },
	{ "&gnapprox;", "\342\252\212" },
	{ "&gne;", "\342\252\210" },
	{ "&gneq;", "\342\252\210" },
	{ "&gneqq;", "\342\211\251" },
	{ "&gnsim;", "\342\213\247" },
	{ "&gopf;", "\355\240\265\355\265\230" },
	{ "&grave;", "\140" },	/* ` */
	{ "&gscr;", "\342\204\212" },
	{ "&gsim;", "\342\211\263" },
	{ "&gsime;", "\342\252\216" },
	{ "&gsiml;", "\342\252\220" },
	{ "&gt;", "\76" },	/* > */
	{ "&gtcc;", "\342\252\247" },
	{ "&gtcir;", "\342\251\272" },
	{ "&gtdot;", "\342\213\227" },
	{ "&gtlPar;", "\342\246\225" },
	{ "&gtquest;", "\342\251\274" },
	{ "&gtrapprox;", "\342\252\206" },
	{ "&gtrarr;", "\342\245\270" },
	{ "&gtrdot;", "\342\213\227" },
	{ "&gtreqless;", "\342\213\233" },
	{ "&gtreqqless;", "\342\252\214" },
	{ "&gtrless;", "\342\211\267" },
	{ "&gtrsim;", "\342\211\263" },
	{ "&gvertneqq;", "\342\211\251\357\270\200" },
	{ "&gvnE;", "\342\211\251\357\270\200" },
	{ "&hArr;", "\342\207\224" },
	{ "&hairsp;", "\342\200\212" },
	{ "&half;", "\302\275" },
	{ "&hamilt;", "\342\204\213" },
	{ "&hardcy;", "\321\212" },
	{ "&harr;", "\342\206\224" },
	{ "&harrcir;", "\342\245\210" },
	{ "&harrw;", "\342\206\255" },
	{ "&hbar;", "\342\204\217" },
	{ "&hcirc;", "\304\245" },
	{ "&hearts;", "\342\231\245" },
	{ "&heartsuit;", "\342\231\245" },
	{ "&hellip;", "\342\200\246" },
	{ "&hercon;", "\342\212\271" },
	{ "&hfr;", "\355\240\265\355\264\245" },
	{ "&hksearow;", "\342\244\245" },
	{ "&hkswarow;", "\342\244\246" },
	{ "&hoarr;", "\342\207\277" },
	{ "&homtht;", "\342\210\273" },
	{ "&hookleftarrow;", "\342\206\251" },
	{ "&hookrightarrow;", "\342\206\252" },
	{ "&hopf;", "\355\240\265\355\265\231" },
	{ "&horbar;", "\342\200\225" },
	{ "&hscr;", "\355\240\265\355\262\275" },
	{ "&hslash;", "\342\204\217" },
	{ "&hstrok;", "\304\247" },
	{ "&hybull;", "\342\201\203" },
	{ "&hyphen;", "\342\200\220" },
	{ "&iacute;", "\303\255" },
	{ "&ic;", "\342\201\243" },
	{ "&icirc;", "\303\256" },
	{ "&icy;", "\320\270" },
	{ "&iecy;", "\320\265" },
	{ "&iexcl;", "\302\241" },
	{ "&iff;", "\342\207\224" },
	{ "&ifr;", "\355\240\265\355\264\246" },
	{ "&igrave;", "\303\254" },
	{ "&ii;", "\342\205\210" },
	{ "&iiiint;", "\342\250\214" },
	{ "&iiint;", "\342\210\255" },
	{ "&iinfin;", "\342\247\234" },
	{ "&iiota;", "\342\204\251" },
	{ "&ijlig;", "\304\263" },
	{ "&imacr;", "\304\253" },
	{ "&image;", "\342\204\221" },
	{ "&imagline;", "\342\204\220" },
	{ "&imagpart;", "\342\204\221" },
	{ "&imath;", "\304\261" },
	{ "&imof;", "\342\212\267" },
	{ "&imped;", "\306\265" },
	{ "&in;", "\342\210\210" },
	{ "&incare;", "\342\204\205" },
	{ "&infin;", "\342\210\236" },
	{ "&infintie;", "\342\247\235" },
	{ "&inodot;", "\304\261" },
	{ "&int;", "\342\210\253" },
	{ "&intcal;", "\342\212\272" },
	{ "&integers;", "\342\204\244" },
	{ "&intercal;", "\342\212\272" },
	{ "&intlarhk;", "\342\250\227" },
	{ "&intprod;", "\342\250\274" },
	{ "&iocy;", "\321\221" },
	{ "&iogon;", "\304\257" },
	{ "&iopf;", "\355\240\265\355\265\232" },
	{ "&iota;", "\316\271" },
	{ "&iprod;", "\342\250\274" },
	{ "&iquest;", "\302\277" },
	{ "&iscr;", "\355\240\265\355\262\276" },
	{ "&isin;", "\342\210\210" },
	{ "&isinE;", "\342\213\271" },
	{ "&isindot;", "\342\213\265" },
	{ "&isins;", "\342\213\264" },
	{ "&isinsv;", "\342\213\263" },
	{ "&isinv;", "\342\210\210" },
	{ "&it;", "\342\201\242" },
	{ "&itilde;", "\304\251" },
	{ "&iukcy;", "\321\226" },
	{ "&iuml;", "\303\257" },
	{ "&jcirc;", "\304\265" },
	{ "&jcy;", "\320\271" },
	{ "&jfr;", "\355\240\265\355\264\247" },
	{ "&jmath;", "\310\267" },
	{ "&jopf;", "\355\240\265\355\265\233" },
	{ "&jscr;", "\355\240\265\355\262\277" },
	{ "&jsercy;", "\321\230" },
	{ "&jukcy;", "\321\224" },
	{ "&kappa;", "\316\272" },
	{ "&kappav;", "\317\260" },
	{ "&kcedil;", "\304\267" },
	{ "&kcy;", "\320\272" },
	{ "&kfr;", "\355\240\265\355\264\250" },
	{ "&kgreen;", "\304\270" },
	{ "&khcy;", "\321\205" },
	{ "&kjcy;", "\321\234" },
	{ "&kopf;", "\355\240\265\355\265\234" },
	{ "&kscr;", "\355\240\265\355\263\200" },
	{ "&lAarr;", "\342\207\232" },
	{ "&lArr;", "\342\207\220" },
	{ "&lAtail;", "\342\244\233" },
	{ "&lBarr;", "\342\244\216" },
	{ "&lE;", "\342\211\246" },
	{ "&lEg;", "\342\252\213" },
	{ "&lHar;", "\342\245\242" },
	{ "&lacute;", "\304\272" },
	{ "&laemptyv;", "\342\246\264" },
	{ "&lagran;", "\342\204\222" },
	{ "&lambda;", "\316\273" },
	{ "&lang;", "\342\237\250" },
	{ "&langd;", "\342\246\221" },
	{ "&langle;", "\342\237\250" },
	{ "&lap;", "\342\252\205" },
	{ "&laquo;", "\302\253" },
	{ "&larr;", "\342\206\220" },
	{ "&larrb;", "\342\207\244" },
	{ "&larrbfs;", "\342\244\237" },
	{ "&larrfs;", "\342\244\235" },
	{ "&larrhk;", "\342\206\251" },
	{ "&larrlp;", "\342\206\253" },
	{ "&larrpl;", "\342\244\271" },
	{ "&larrsim;", "\342\245\263" },
	{ "&larrtl;", "\342\206\242" },
	{ "&lat;", "\342\252\253" },
	{ "&latail;", "\342\244\231" },
	{ "&late;", "\342\252\255" },
	{ "&lates;", "\342\252\255\357\270\200" },
	{ "&lbarr;", "\342\244\214" },
	{ "&lbbrk;", "\342\235\262" },
	{ "&lbrace;", "\173" },	/* { */
	{ "&lbrack;", "\133" },	/* [ */
	{ "&lbrke;", "\342\246\213" },
	{ "&lbrksld;", "\342\246\217" },
	{ "&lbrkslu;", "\342\246\215" },
	{ "&lcaron;", "\304\276" },
	{ "&lcedil;", "\304\274" },
	{ "&lceil;", "\342\214\210" },
	{ "&lcub;", "\173" },	/* { */
	{ "&lcy;", "\320\273" },
	{ "&ldca;", "\342\244\266" },
	{ "&ldquo;", "\342\200\234" },
	{ "&ldquor;", "\342\200\236" },
	{ "&ldrdhar;", "\342\245\247" },
	{ "&ldrushar;", "\342\245\213" },
	{ "&ldsh;", "\342\206\262" },
	{ "&le;", "\342\211\244" },
	{ "&leftarrow;", "\342\206\220" },
	{ "&leftarrowtail;", "\342\206\242" },
	{ "&leftharpoondown;", "\342\206\275" },
	{ "&leftharpoonup;", "\342\206\274" },
	{ "&leftleftarrows;", "\342\207\207" },
	{ "&leftrightarrow;", "\342\206\224" },
	{ "&leftrightarrows;", "\342\207\206" },
	{ "&leftrightharpoons;", "\342\207\213" },
	{ "&leftrightsquigarrow;", "\342\206\255" },
	{ "&leftthreetimes;", "\342\213\213" },
	{ "&leg;", "\342\213\232" },
	{ "&leq;", "\342\211\244" },
	{ "&leqq;", "\342\211\246" },
	{ "&leqslant;", "\342\251\275" },
	{ "&les;", "\342\251\275" },
	{ "&lescc;", "\342\252\250" },
	{ "&lesdot;", "\342\251\277" },
	{ "&lesdoto;", "\342\252\201" },
	{ "&lesdotor;", "\342\252\203" },
	{ "&lesg;", "\342\213\232\357\270\200" },
	{ "&lesges;", "\342\252\223" },
	{ "&lessapprox;", "\342\252\205" },
	{ "&lessdot;", "\342\213\226" },
	{ "&lesseqgtr;", "\342\213\232" },
	{ "&lesseqqgtr;", "\342\252\213" },
	{ "&lessgtr;", "\342\211\266" },
	{ "&lesssim;", "\342\211\262" },
	{ "&lfisht;", "\342\245\274" },
	{ "&lfloor;", "\342\214\212" },
	{ "&lfr;", "\355\240\265\355\264\251" },
	{ "&lg;", "\342\211\266" },
	{ "&lgE;", "\342\252\221" },
	{ "&lhard;", "\342\206\275" },
	{ "&lharu;", "\342\206\274" },
	{ "&lharul;", "\342\245\252" },
	{ "&lhblk;", "\342\226\204" },
	{ "&ljcy;", "\321\231" },
	{ "&ll;", "\342\211\252" },
	{ "&llarr;", "\342\207\207" },
	{ "&llcorner;", "\342\214\236" },
	{ "&llhard;", "\342\245\253" },
	{ "&lltri;", "\342\227\272" },
	{ "&lmidot;", "\305\200" },
	{ "&lmoust;", "\342\216\260" },
	{ "&lmoustache;", "\342\216\260" },
	{ "&lnE;", "\342\211\250" },
	{ "&lnap;", "\342\252\211" },
	{ "&lnapprox;", "\342\252\211" },
	{ "&lne;", "\342\252\207" },
	{ "&lneq;", "\342\252\207" },
	{ "&lneqq;", "\342\211\250" },
	{ "&lnsim;", "\342\213\246" },
	{ "&loang;", "\342\237\254" },
	{ "&loarr;", "\342\207\275" },
	{ "&lobrk;", "\342\237\246" },
	{ "&longleftarrow;", "\342\237\265" },
	{ "&longleftrightarrow;", "\342\237\267" },
	{ "&longmapsto;", "\342\237\274" },
	{ "&longrightarrow;", "\342\237\266" },
	{ "&looparrowleft;", "\342\206\253" },
	{ "&looparrowright;", "\342\206\254" },
	{ "&lopar;", "\342\246\205" },
	{ "&lopf;", "\355\240\265\355\265\235" },
	{ "&loplus;", "\342\250\255" },
	{ "&lotimes;", "\342\250\264" },
	{ "&lowast;", "\342\210\227" },
	{ "&lowbar;", "\137" },	/* _ */
	{ "&loz;", "\342\227\212" },
	{ "&lozenge;", "\342\227\212" },
	{ "&lozf;", "\342\247\253" },
	{ "&lpar;", "\50" },	/* ( */
	{ "&lparlt;", "\342\246\223" },
	{ "&lrarr;", "\342\207\206" },
	{ "&lrcorner;", "\342\214\237" },
	{ "&lrhar;", "\342\207\213" },
	{ "&lrhard;", "\342\245\255" },
	{ "&lrm;", "\342\200\216" },
	{ "&lrtri;", "\342\212\277" },
	{ "&lsaquo;", "\342\200\271" },
	{ "&lscr;", "\355\240\265\355\263\201" },
	{ "&lsh;", "\342\206\260" },
	{ "&lsim;", "\342\211\262" },
	{ "&lsime;", "\342\252\215" },
	{ "&lsimg;", "\342\252\217" },
	{ "&lsqb;", "\133" },	/* [ */
	{ "&lsquo;", "\342\200\230" },
	{ "&lsquor;", "\342\200\232" },
	{ "&lstrok;", "\305\202" },
	{ "&lt;", "\74" },	/* < */
	{ "&ltcc;", "\342\252\246" },
	{ "&ltcir;", "\342\251\271" },
	{ "&ltdot;", "\342\213\226" },
	{ "&lthree;", "\342\213\213" },
	{ "&ltimes;", "\342\213\211" },
	{ "&ltlarr;", "\342\245\266" },
	{ "&ltquest;", "\342\251\273" },
	{ "&ltrPar;", "\342\246\226" },
	{ "&ltri;", "\342\227\203" },
	{ "&ltrie;", "\342\212\264" },
	{ "&ltrif;", "\342\227\202" },
	{ "&lurdshar;", "\342\245\212" },
	{ "&luruhar;", "\342\245\246" },
	{ "&lvertneqq;", "\342\211\250\357\270\200" },
	{ "&lvnE;", "\342\211\250\357\270\200" },
	{ "&mDDot;", "\342\210\272" },
	{ "&macr;", "\302\257" },
	{ "&male;", "\342\231\202" },
	{ "&malt;", "\342\234\240" },
	{ "&maltese;", "\342\234\240" },
	{ "&map;", "\342\206\246" },
	{ "&mapsto;", "\342\206\246" },
	{ "&mapstodown;", "\342\206\247" },
	{ "&mapstoleft;", "\342\206\244" },
	{ "&mapstoup;", "\342\206\245" },
	{ "&marker;", "\342\226\256" },
	{ "&mcomma;", "\342\250\251" },
	{ "&mcy;", "\320\274" },
	{ "&mdash;", "\342\200\224" },
	{ "&measuredangle;", "\342\210\241" },
	{ "&mfr;", "\355\240\265\355\264\252" },
	{ "&mho;", "\342\204\247" },
	{ "&micro;", "\302\265" },
	{ "&mid;", "\342\210\243" },
	{ "&midast;", "\52" },	/* * */
	{ "&midcir;", "\342\253\260" },
	{ "&middot;", "\302\267" },
	{ "&minus;", "\342\210\222" },
	{ "&minusb;", "\342\212\237" },
	{ "&minusd;", "\342\210\270" },
	{ "&minusdu;", "\342\250\252" },
	{ "&mlcp;", "\342\253\233" },
	{ "&mldr;", "\342\200\246" },
	{ "&mnplus;", "\342\210\223" },
	{ "&models;", "\342\212\247" },
	{ "&mopf;", "\355\240\265\355\265\236" },
	{ "&mp;", "\342\210\223" },
	{ "&mscr;", "\355\240\265\355\263\202" },
	{ "&mstpos;", "\342\210\276" },
	{ "&mu;", "\316\274" },
	{ "&multimap;", "\342\212\270" },
	{ "&mumap;", "\342\212\270" },
	{ "&nGg;", "\342\213\231\314\270" },
	{ "&nGt;", "\342\211\253\342\203\222" },
	{ "&nGtv;", "\342\211\253\314\270" },
	{ "&nLeftarrow;", "\342\207\215" },
	{ "&nLeftrightarrow;", "\342\207\216" },
	{ "&nLl;", "\342\213\230\314\270" },
	{ "&nLt;", "\342\211\252\342\203\222" },
	{ "&nLtv;", "\342\211\252\314\270" },
	{ "&nRightarrow;", "\342\207\217" },
	{ "&nVDash;", "\342\212\257" },
	{ "&nVdash;", "\342\212\256" },
	{ "&nabla;", "\342\210\207" },
	{ "&nacute;", "\305\204" },
	{ "&nang;", "\342\210\240\342\203\222" },
	{ "&nap;", "\342\211\211" },
	{ "&napE;", "\342\251\260\314\270" },
	{ "&napid;", "\342\211\213\314\270" },
	{ "&napos;", "\305\211" },
	{ "&napprox;", "\342\211\211" },
	{ "&natur;", "\342\231\256" },
	{ "&natural;", "\342\231\256" },
	{ "&naturals;", "\342\204\225" },
	{ "&nbsp;", "\302\240" },
	{ "&nbump;", "\342\211\216\314\270" },
	{ "&nbumpe;", "\342\211\217\314\270" },
	{ "&ncap;", "\342\251\203" },
	{ "&ncaron;", "\305\210" },
	{ "&ncedil;", "\305\206" },
	{ "&ncong;", "\342\211\207" },
	{ "&ncongdot;", "\342\251\255\314\270" },
	{ "&ncup;", "\342\251\202" },
	{ "&ncy;", "\320\275" },
	{ "&ndash;", "\342\200\223" },
	{ "&ne;", "\342\211\240" },
	{ "&neArr;", "\342\207\227" },
	{ "&nearhk;", "\342\244\244" },
	{ "&nearr;", "\342\206\227" },
	{ "&nearrow;", "\342\206\227" },
	{ "&nedot;", "\342\211\220\314\270" },
	{ "&nequiv;", "\342\211\242" },
	{ "&nesear;", "\342\244\250" },
	{ "&nesim;", "\342\211\202\314\270" },
	{ "&nexist;", "\342\210\204" },
	{ "&nexists;", "\342\210\204" },
	{ "&nfr;", "\355\240\265\355\264\253" },
	{ "&ngE;", "\342\211\247\314\270" },
	{ "&nge;", "\342\211\261" },
	{ "&ngeq;", "\342\211\261" },
	{ "&ngeqq;", "\342\211\247\314\270" },
	{ "&ngeqslant;", "\342\251\276\314\270" },
	{ "&nges;", "\342\251\276\314\270" },
	{ "&ngsim;", "\342\211\265" },
	{ "&ngt;", "\342\211\257" },
	{ "&ngtr;", "\342\211\257" },
	{ "&nhArr;", "\342\207\216" },
	{ "&nharr;", "\342\206\256" },
	{ "&nhpar;", "\342\253\262" },
	{ "&ni;", "\342\210\213" },
	{ "&nis;", "\342\213\274" },
	{ "&nisd;", "\342\213\272" },
	{ "&niv;", "\342\210\213" },
	{ "&njcy;", "\321\232" },
	{ "&nlArr;", "\342\207\215" },
	{ "&nlE;", "\342\211\246\314\270" },
	{ "&nlarr;", "\342\206\232" },
	{ "&nldr;", "\342\200\245" },
	{ "&nle;", "\342\211\260" },
	{ "&nleftarrow;", "\342\206\232" },
	{ "&nleftrightarrow;", "\342\206\256" },
	{ "&nleq;", "\342\211\260" },
	{ "&nleqq;", "\342\211\246\314\270" },
	{ "&nleqslant;", "\342\251\275\314\270" },
	{ "&nles;", "\342\251\275\314\270" },
	{ "&nless;", "\342\211\256" },
	{ "&nlsim;", "\342\211\264" },
	{ "&nlt;", "\342\211\256" },
	{ "&nltri;", "\342\213\252" },
	{ "&nltrie;", "\342\213\254" },
	{ "&nmid;", "\342\210\244" },
	{ "&nopf;", "\355\240\265\355\265\237" },
	{ "&not;", "\302\254" },
	{ "&notin;", "\342\210\211" },
	{ "&notinE;", "\342\213\271\314\270" },
	{ "&notindot;", "\342\213\265\314\270" },
	{ "&notinva;", "\342\210\211" },
	{ "&notinvb;", "\342\213\267" },
	{ "&notinvc;", "\342\213\266" },
	{ "&notni;", "\342\210\214" },
	{ "&notniva;", "\342\210\214" },
	{ "&notnivb;", "\342\213\276" },
	{ "&notnivc;", "\342\213\275" },
	{ "&npar;", "\342\210\246" },
	{ "&nparallel;", "\342\210\246" },
	{ "&nparsl;", "\342\253\275\342\203\245" },
	{ "&npart;", "\342\210\202\314\270" },
	{ "&npolint;", "\342\250\224" },
	{ "&npr;", "\342\212\200" },
	{ "&nprcue;", "\342\213\240" },
	{ "&npre;", "\342\252\257\314\270" },
	{ "&nprec;", "\342\212\200" },
	{ "&npreceq;", "\342\252\257\314\270" },
	{ "&nrArr;", "\342\207\217" },
	{ "&nrarr;", "\342\206\233" },
	{ "&nrarrc;", "\342\244\263\314\270" },
	{ "&nrarrw;", "\342\206\235\314\270" },
	{ "&nrightarrow;", "\342\206\233" },
	{ "&nrtri;", "\342\213\253" },
	{ "&nrtrie;", "\342\213\255" },
	{ "&nsc;", "\342\212\201" },
	{ "&nsccue;", "\342\213\241" },
	{ "&nsce;", "\342\252\260\314\270" },
	{ "&nscr;", "\355\240\265\355\263\203" },
	{ "&nshortmid;", "\342\210\244" },
	{ "&nshortparallel;", "\342\210\246" },
	{ "&nsim;", "\342\211\201" },
	{ "&nsime;", "\342\211\204" },
	{ "&nsimeq;", "\342\211\204" },
	{ "&nsmid;", "\342\210\244" },
	{ "&nspar;", "\342\210\246" },
	{ "&nsqsube;", "\342\213\242" },
	{ "&nsqsupe;", "\342\213\243" },
	{ "&nsub;", "\342\212\204" },
	{ "&nsubE;", "\342\253\205\314\270" },
	{ "&nsube;", "\342\212\210" },
	{ "&nsubset;", "\342\212\202\342\203\222" },
	{ "&nsubseteq;", "\342\212\210" },
	{ "&nsubseteqq;", "\342\253\205\314\270" },
	{ "&nsucc;", "\342\212\201" },
	{ "&nsucceq;", "\342\252\260\314\270" },
	{ "&nsup;", "\342\212\205" },
	{ "&nsupE;", "\342\253\206\314\270" },
	{ "&nsupe;", "\342\212\211" },
	{ "&nsupset;", "\342\212\203\342\203\222" },
	{ "&nsupseteq;", "\342\212\211" },
	{ "&nsupseteqq;", "\342\253\206\314\270" },
	{ "&ntgl;", "\342\211\271" },
	{ "&ntilde;", "\303\261" },
	{ "&ntlg;", "\342\211\270" },
	{ "&ntriangleleft;", "\342\213\252" },
	{ "&ntrianglelefteq;", "\342\213\254" },
	{ "&ntriangleright;", "\342\213\253" },
	{ "&ntrianglerighteq;", "\342\213\255" },
	{ "&nu;", "\316\275" },
	{ "&num;", "\43" },	/* # */
	{ "&numero;", "\342\204\226" },
	{ "&numsp;", "\342\200\207" },
	{ "&nvDash;", "\342\212\255" },
	{ "&nvHarr;", "\342\244\204" },
	{ "&nvap;", "\342\211\215\342\203\222" },
	{ "&nvdash;", "\342\212\254" },
	{ "&nvge;", "\342\211\245\342\203\222" },
	{ "&nvgt;", "\76\342\203\222" },
	{ "&nvinfin;", "\342\247\236" },
	{ "&nvlArr;", "\342\244\202" },
	{ "&nvle;", "\342\211\244\342\203\222" },
	{ "&nvlt;", "\74\342\203\222" },
	{ "&nvltrie;", "\342\212\264\342\203\222" },
	{ "&nvrArr;", "\342\244\203" },
	{ "&nvrtrie;", "\342\212\265\342\203\222" },
	{ "&nvsim;", "\342\210\274\342\203\222" },
	{ "&nwArr;", "\342\207\226" },
	{ "&nwarhk;", "\342\244\243" },
	{ "&nwarr;", "\342\206\226" },
	{ "&nwarrow;", "\342\206\226" },
	{ "&nwnear;", "\342\244\247" },
	{ "&oS;", "\342\223\210" },
	{ "&oacute;", "\303\263" },
	{ "&oast;", "\342\212\233" },
	{ "&ocir;", "\342\212\232" },
	{ "&ocirc;", "\303\264" },
	{ "&ocy;", "\320\276" },
	{ "&odash;", "\342\212\235" },
	{ "&odblac;", "\305\221" },
	{ "&odiv;", "\342\250\270" },
	{ "&odot;", "\342\212\231" },
	{ "&odsold;", "\342\246\274" },
	{ "&oelig;", "\305\223" },
	{ "&ofcir;", "\342\246\277" },
	{ "&ofr;", "\355\240\265\355\264\254" },
	{ "&ogon;", "\313\233" },
	{ "&ograve;", "\303\262" },
	{ "&ogt;", "\342\247\201" },
	{ "&ohbar;", "\342\246\265" },
	{ "&ohm;", "\316\251" },
	{ "&oint;", "\342\210\256" },
	{ "&olarr;", "\342\206\272" },
	{ "&olcir;", "\342\246\276" },
	{ "&olcross;", "\342\246\273" },
	{ "&oline;", "\342\200\276" },
	{ "&olt;", "\342\247\200" },
	{ "&omacr;", "\305\215" },
	{ "&omega;", "\317\211" },
	{ "&omicron;", "\316\277" },
	{ "&omid;", "\342\246\266" },
	{ "&ominus;", "\342\212\226" },
	{ "&oopf;", "\355\240\265\355\265\240" },
	{ "&opar;", "\342\246\267" },
	{ "&operp;", "\342\246\271" },
	{ "&oplus;", "\342\212\225" },
	{ "&or;", "\342\210\250" },
	{ "&orarr;", "\342\206\273" },
	{ "&ord;", "\342\251\235" },
	{ "&order;", "\342\204\264" },
	{ "&orderof;", "\342\204\264" },
	{ "&ordf;", "\302\252" },
	{ "&ordm;", "\302\272" },
	{ "&origof;", "\342\212\266" },
	{ "&oror;", "\342\251\226" },
	{ "&orslope;", "\342\251\227" },
	{ "&orv;", "\342\251\233" },
	{ "&oscr;", "\342\204\264" },
	{ "&oslash;", "\303\270" },
	{ "&osol;", "\342\212\230" },
	{ "&otilde;", "\303\265" },
	{ "&otimes;", "\342\212\227" },
	{ "&otimesas;", "\342\250\266" },
	{ "&ouml;", "\303\266" },
	{ "&ovbar;", "\342\214\275" },
	{ "&par;", "\342\210\245" },
	{ "&para;", "\302\266" },
	{ "&parallel;", "\342\210\245" },
	{ "&parsim;", "\342\253\263" },
	{ "&parsl;", "\342\253\275" },
	{ "&part;", "\342\210\202" },
	{ "&pcy;", "\320\277" },
	{ "&percnt;", "\45" },	/* % */
	{ "&period;", "\56" },	/* . */
	{ "&permil;", "\342\200\260" },
	{ "&perp;", "\342\212\245" },
	{ "&pertenk;", "\342\200\261" },
	{ "&pfr;", "\355\240\265\355\264\255" },
	{ "&phi;", "\317\206" },
	{ "&phiv;", "\317\225" },
	{ "&phmmat;", "\342\204\263" },
	{ "&phone;", "\342\230\216" },
	{ "&pi;", "\317\200" },
	{ "&pitchfork;", "\342\213\224" },
	{ "&piv;", "\317\226" },
	{ "&planck;", "\342\204\217" },
	{ "&planckh;", "\342\204\216" },
	{ "&plankv;", "\342\204\217" },
	{ "&plus;", "\53" },	/* + */
	{ "&plusacir;", "\342\250\243" },
	{ "&plusb;", "\342\212\236" },
	{ "&pluscir;", "\342\250\242" },
	{ "&plusdo;", "\342\210\224" },
	{ "&plusdu;", "\342\250\245" },
	{ "&pluse;", "\342\251\262" },
	{ "&plusmn;", "\302\261" },
	{ "&plussim;", "\342\250\246" },
	{ "&plustwo;", "\342\250\247" },
	{ "&pm;", "\302\261" },
	{ "&pointint;", "\342\250\225" },
	{ "&popf;", "\355\240\265\355\265\241" },
	{ "&pound;", "\302\243" },
	{ "&pr;", "\342\211\272" },
	{ "&prE;", "\342\252\263" },
	{ "&prap;", "\342\252\267" },
	{ "&prcue;", "\342\211\274" },
	{ "&pre;", "\342\252\257" },
	{ "&prec;", "\342\211\272" },
	{ "&precapprox;", "\342\252\267" },
	{ "&preccurlyeq;", "\342\211\274" },
	{ "&preceq;", "\342\252\257" },
	{ "&precnapprox;", "\342\252\271" },
	{ "&precneqq;", "\342\252\265" },
	{ "&precnsim;", "\342\213\250" },
	{ "&precsim;", "\342\211\276" },
	{ "&prime;", "\342\200\262" },
	{ "&primes;", "\342\204\231" },
	{ "&prnE;", "\342\252\265" },
	{ "&prnap;", "\342\252\271" },
	{ "&prnsim;", "\342\213\250" },
	{ "&prod;", "\342\210\217" },
	{ "&profalar;", "\342\214\256" },
	{ "&profline;", "\342\214\222" },
	{ "&profsurf;", "\342\214\223" },
	{ "&prop;", "\342\210\235" },
	{ "&propto;", "\342\210\235" },
	{ "&prsim;", "\342\211\276" },
	{ "&prurel;", "\342\212\260" },
	{ "&pscr;", "\355\240\265\355\263\205" },
	{ "&psi;", "\317\210" },
	{ "&puncsp;", "\342\200\210" },
	{ "&qfr;", "\355\240\265\355\264\256" },
	{ "&qint;", "\342\250\214" },
	{ "&qopf;", "\355\240\265\355\265\242" },
	{ "&qprime;", "\342\201\227" },
	{ "&qscr;", "\355\240\265\355\263\206" },
	{ "&quaternions;", "\342\204\215" },
	{ "&quatint;", "\342\250\226" },
	{ "&quest;", "\77" },	/* ? */
	{ "&questeq;", "\342\211\237" },
	{ "&quot;", "\42" },	/* " */
	{ "&rAarr;", "\342\207\233" },
	{ "&rArr;", "\342\207\222" },
	{ "&rAtail;", "\342\244\234" },
	{ "&rBarr;", "\342\244\217" },
	{ "&rHar;", "\342\245\244" },
	{ "&race;", "\342\210\275\314\261" },
	{ "&racute;", "\305\225" },
	{ "&radic;", "\342\210\232" },
	{ "&raemptyv;", "\342\246\263" },
	{ "&rang;", "\342\237\251" },
	{ "&rangd;", "\342\246\222" },
	{ "&range;", "\342\246\245" },
	{ "&rangle;", "\342\237\251" },
	{ "&raquo;", "\302\273" },
	{ "&rarr;", "\342\206\222" },
	{ "&rarrap;", "\342\245\265" },
	{ "&rarrb;", "\342\207\245" },
	{ "&rarrbfs;", "\342\244\240" },
	{ "&rarrc;", "\342\244\263" },
	{ "&rarrfs;", "\342\244\236" },
	{ "&rarrhk;", "\342\206\252" },
	{ "&rarrlp;", "\342\206\254" },
	{ "&rarrpl;", "\342\245\205" },
	{ "&rarrsim;", "\342\245\264" },
	{ "&rarrtl;", "\342\206\243" },
	{ "&rarrw;", "\342\206\235" },
	{ "&ratail;", "\342\244\232" },
	{ "&ratio;", "\342\210\266" },
	{ "&rationals;", "\342\204\232" },
	{ "&rbarr;", "\342\244\215" },
	{ "&rbbrk;", "\342\235\263" },
	{ "&rbrace;", "\175" },	/* } */
	{ "&rbrack;", "\135" },	/* ] */
	{ "&rbrke;", "\342\246\214" },
	{ "&rbrksld;", "\342\246\216" },
	{ "&rbrkslu;", "\342\246\220" },
	{ "&rcaron;", "\305\231" },
	{ "&rcedil;", "\305\227" },
	{ "&rceil;", "\342\214\211" },
	{ "&rcub;", "\175" },	/* } */
	{ "&rcy;", "\321\200" },
	{ "&rdca;", "\342\244\267" },
	{ "&rdldhar;", "\342\245\251" },
	{ "&rdquo;", "\342\200\235" },
	{ "&rdquor;", "\342\200\235" },
	{ "&rdsh;", "\342\206\263" },
	{ "&real;", "\342\204\234" },
	{ "&realine;", "\342\204\233" },
	{ "&realpart;", "\342\204\234" },
	{ "&reals;", "\342\204\235" },
	{ "&rect;", "\342\226\255" },
	{ "&reg;", "\302\256" },
	{ "&rfisht;", "\342\245\275" },
	{ "&rfloor;", "\342\214\213" },
	{ "&rfr;", "\355\240\265\355\264\257" },
	{ "&rhard;", "\342\207\201" },
	{ "&rharu;", "\342\207\200" },
	{ "&rharul;", "\342\245\254" },
	{ "&rho;", "\317\201" },
	{ "&rhov;", "\317\261" },
	{ "&rightarrow;", "\342\206\222" },
	{ "&rightarrowtail;", "\342\206\243" },
	{ "&rightharpoondown;", "\342\207\201" },
	{ "&rightharpoonup;", "\342\207\200" },
	{ "&rightleftarrows;", "\342\207\204" },
	{ "&rightleftharpoons;", "\342\207\214" },
	{ "&rightrightarrows;", "\342\207\211" },
	{ "&rightsquigarrow;", "\342\206\235" },
	{ "&rightthreetimes;", "\342\213\214" },
	{ "&ring;", "\313\232" },
	{ "&risingdotseq;", "\342\211\223" },
	{ "&rlarr;", "\342\207\204" },
	{ "&rlhar;", "\342\207\214" },
	{ "&rlm;", "\342\200\217" },
	{ "&rmoust;", "\342\216\261" },
	{ "&rmoustache;", "\342\216\261" },
	{ "&rnmid;", "\342\253\256" },
	{ "&roang;", "\342\237\255" },
	{ "&roarr;", "\342\207\276" },
	{ "&robrk;", "\342\237\247" },
	{ "&ropar;", "\342\246\206" },
	{ "&ropf;", "\355\240\265\355\265\243" },
	{ "&roplus;", "\342\250\256" },
	{ "&rotimes;", "\342\250\265" },
	{ "&rpar;", "\51" },	/* ) */
	{ "&rpargt;", "\342\246\224" },
	{ "&rppolint;", "\342\250\222" },
	{ "&rrarr;", "\342\207\211" },
	{ "&rsaquo;", "\342\200\272" },
	{ "&rscr;", "\355\240\265\355\263\207" },
	{ "&rsh;", "\342\206\261" },
	{ "&rsqb;", "\135" },	/* ] */
	{ "&rsquo;", "\342\200\231" },
	{ "&rsquor;", "\342\200\231" },
	{ "&rthree;", "\342\213\214" },
	{ "&rtimes;", "\342\213\212" },
	{ "&rtri;", "\342\226\271" },
	{ "&rtrie;", "\342\212\265" },
	{ "&rtrif;", "\342\226\270" },
	{ "&rtriltri;", "\342\247\216" },
	{ "&ruluhar;", "\342\245\250" },
	{ "&rx;", "\342\204\236" },
	{ "&sacute;", "\305\233" },
	{ "&sbquo;", "\342\200\232" },
	{ "&sc;", "\342\211\273" },
	{ "&scE;", "\342\252\264" },
	{ "&scap;", "\342\252\270" },
	{ "&scaron;", "\305\241" },
	{ "&sccue;", "\342\211\275" },
	{ "&sce;", "\342\252\260" },
	{ "&scedil;", "\305\237" },
	{ "&scirc;", "\305\235" },
	{ "&scnE;", "\342\252\266" },
	{ "&scnap;", "\342\252\272" },
	{ "&scnsim;", "\342\213\251" },
	{ "&scpolint;", "\342\250\223" },
	{ "&scsim;", "\342\211\277" },
	{ "&scy;", "\321\201" },
	{ "&sdot;", "\342\213\205" },
	{ "&sdotb;", "\342\212\241" },
	{ "&sdote;", "\342\251\246" },
	{ "&seArr;", "\342\207\230" },
	{ "&searhk;", "\342\244\245" },
	{ "&searr;", "\342\206\230" },
	{ "&searrow;", "\342\206\230" },
	{ "&sect;", "\302\247" },
	{ "&semi;", "\73" },	/* ; */
	{ "&seswar;", "\342\244\251" },
	{ "&setminus;", "\342\210\226" },
	{ "&setmn;", "\342\210\226" },
	{ "&sext;", "\342\234\266" },
	{ "&sfr;", "\355\240\265\355\264\260" },
	{ "&sfrown;", "\342\214\242" },
	{ "&sharp;", "\342\231\257" },
	{ "&shchcy;", "\321\211" },
	{ "&shcy;", "\321\210" },
	{ "&shortmid;", "\342\210\243" },
	{ "&shortparallel;", "\342\210\245" },
	{ "&shy;", "\302\255" },
	{ "&sigma;", "\317\203" },
	{ "&sigmaf;", "\317\202" },
	{ "&sigmav;", "\317\202" },
	{ "&sim;", "\342\210\274" },
	{ "&simdot;", "\342\251\252" },
	{ "&sime;", "\342\211\203" },
	{ "&simeq;", "\342\211\203" },
	{ "&simg;", "\342\252\236" },
	{ "&simgE;", "\342\252\240" },
	{ "&siml;", "\342\252\235" },
	{ "&simlE;", "\342\252\237" },
	{ "&simne;", "\342\211\206" },
	{ "&simplus;", "\342\250\244" },
	{ "&simrarr;", "\342\245\262" },
	{ "&slarr;", "\342\206\220" },
	{ "&smallsetminus;", "\342\210\226" },
	{ "&smashp;", "\342\250\263" },
	{ "&smeparsl;", "\342\247\244" },
	{ "&smid;", "\342\210\243" },
	{ "&smile;", "\342\214\243" },
	{ "&smt;", "\342\252\252" },
	{ "&smte;", "\342\252\254" },
	{ "&smtes;", "\342\252\254\357\270\200" },
	{ "&softcy;", "\321\214" },
	{ "&sol;", "\57" },	/* / */
	{ "&solb;", "\342\247\204" },
	{ "&solbar;", "\342\214\277" },
	{ "&sopf;", "\355\240\265\355\265\244" },
	{ "&spades;", "\342\231\240" },
	{ "&spadesuit;", "\342\231\240" },
	{ "&spar;", "\342\210\245" },
	{ "&sqcap;", "\342\212\223" },
	{ "&sqcaps;", "\342\212\223\357\270\200" },
	{ "&sqcup;", "\342\212\224" },
	{ "&sqcups;", "\342\212\224\357\270\200" },
	{ "&sqsub;", "\342\212\217" },
	{ "&sqsube;", "\342\212\221" },
	{ "&sqsubset;", "\342\212\217" },
	{ "&sqsubseteq;", "\342\212\221" },
	{ "&sqsup;", "\342\212\220" },
	{ "&sqsupe;", "\342\212\222" },
	{ "&sqsupset;", "\342\212\220" },
	{ "&sqsupseteq;", "\342\212\222" },
	{ "&squ;", "\342\226\241" },
	{ "&square;", "\342\226\241" },
	{ "&squarf;", "\342\226\252" },
	{ "&squf;", "\342\226\252" },
	{ "&srarr;", "\342\206\222" },
	{ "&sscr;", "\355\240\265\355\263\210" },
	{ "&ssetmn;", "\342\210\226" },
	{ "&ssmile;", "\342\214\243" },
	{ "&sstarf;", "\342\213\206" },
	{ "&star;", "\342\230\206" },
	{ "&starf;", "\342\230\205" },
	{ "&straightepsilon;", "\317\265" },
	{ "&straightphi;", "\317\225" },
	{ "&strns;", "\302\257" },
	{ "&sub;", "\342\212\202" },
	{ "&subE;", "\342\253\205" },
	{ "&subdot;", "\342\252\275" },
	{ "&sube;", "\342\212\206" },
	{ "&subedot;", "\342\253\203" },
	{ "&submult;", "\342\253\201" },
	{ "&subnE;", "\342\253\213" },
	{ "&subne;", "\342\212\212" },
	{ "&subplus;", "\342\252\277" },
	{ "&subrarr;", "\342\245\271" },
	{ "&subset;", "\342\212\202" },
	{ "&subseteq;", "\342\212\206" },
	{ "&subseteqq;", "\342\253\205" },
	{ "&subsetneq;", "\342\212\212" },
	{ "&subsetneqq;", "\342\253\213" },
	{ "&subsim;", "\342\253\207" },
	{ "&subsub;", "\342\253\225" },
	{ "&subsup;", "\342\253\223" },
	{ "&succ;", "\342\211\273" },
	{ "&succapprox;", "\342\252\270" },
	{ "&succcurlyeq;", "\342\211\275" },
	{ "&succeq;", "\342\252\260" },
	{ "&succnapprox;", "\342\252\272" },
	{ "&succneqq;", "\342\252\266" },
	{ "&succnsim;", "\342\213\251" },
	{ "&succsim;", "\342\211\277" },
	{ "&sum;", "\342\210\221" },
	{ "&sung;", "\342\231\252" },
	{ "&sup1;", "\302\271" },
	{ "&sup2;", "\302\262" },
	{ "&sup3;", "\302\263" },
	{ "&sup;", "\342\212\203" },
	{ "&supE;", "\342\253\206" },
	{ "&supdot;", "\342\252\276" },
	{ "&supdsub;", "\342\253\230" },
	{ "&supe;", "\342\212\207" },
	{ "&supedot;", "\342\253\204" },
	{ "&suphsol;", "\342\237\211" },
	{ "&suphsub;", "\342\253\227" },
	{ "&suplarr;", "\342\245\273" },
	{ "&supmult;", "\342\253\202" },
	{ "&supnE;", "\342\253\214" },
	{ "&supne;", "\342\212\213" },
	{ "&supplus;", "\342\253\200" },
	{ "&supset;", "\342\212\203" },
	{ "&supseteq;", "\342\212\207" },
	{ "&supseteqq;", "\342\253\206" },
	{ "&supsetneq;", "\342\212\213" },
	{ "&supsetneqq;", "\342\253\214" },
	{ "&supsim;", "\342\253\210" },
	{ "&supsub;", "\342\253\224" },
	{ "&supsup;", "\342\253\226" },
	{ "&swArr;", "\342\207\231" },
	{ "&swarhk;", "\342\244\246" },
	{ "&swarr;", "\342\206\231" },
	{ "&swarrow;", "\342\206\231" },
	{ "&swnwar;", "\342\244\252" },
	{ "&szlig;", "\303\237" },
	{ "&target;", "\342\214\226" },
	{ "&tau;", "\317\204" },
	{ "&tbrk;", "\342\216\264" },
	{ "&tcaron;", "\305\245" },
	{ "&tcedil;", "\305\243" },
	{ "&tcy;", "\321\202" },
	{ "&tdot;", "\342\203\233" },
	{ "&telrec;", "\342\214\225" },
	{ "&tfr;", "\355\240\265\355\264\261" },
	{ "&there4;", "\342\210\264" },
	{ "&therefore;", "\342\210\264" },
	{ "&theta;", "\316\270" },
	{ "&thetasym;", "\317\221" },
	{ "&thetav;", "\317\221" },
	{ "&thickapprox;", "\342\211\210" },
	{ "&thicksim;", "\342\210\274" },
	{ "&thinsp;", "\342\200\211" },
	{ "&thkap;", "\342\211\210" },
	{ "&thksim;", "\342\210\274" },
	{ "&thorn;", "\303\276" },
	{ "&tilde;", "\313\234" },
	{ "&times;", "\303\227" },
	{ "&timesb;", "\342\212\240" },
	{ "&timesbar;", "\342\250\261" },
	{ "&timesd;", "\342\250\260" },
	{ "&tint;", "\342\210\255" },
	{ "&toea;", "\342\244\250" },
	{ "&top;", "\342\212\244" },
	{ "&topbot;", "\342\214\266" },
	{ "&topcir;", "\342\253\261" },
	{ "&topf;", "\355\240\265\355\265\245" },
	{ "&topfork;", "\342\253\232" },
	{ "&tosa;", "\342\244\251" },
	{ "&tprime;", "\342\200\264" },
	{ "&trade;", "\342\204\242" },
	{ "&triangle;", "\342\226\265" },
	{ "&triangledown;", "\342\226\277" },
	{ "&triangleleft;", "\342\227\203" },
	{ "&trianglelefteq;", "\342\212\264" },
	{ "&triangleq;", "\342\211\234" },
	{ "&triangleright;", "\342\226\271" },
	{ "&trianglerighteq;", "\342\212\265" },
	{ "&tridot;", "\342\227\254" },
	{ "&trie;", "\342\211\234" },
	{ "&triminus;", "\342\250\272" },
	{ "&triplus;", "\342\250\271" },
	{ "&trisb;", "\342\247\215" },
	{ "&tritime;", "\342\250\273" },
	{ "&trpezium;", "\342\217\242" },
	{ "&tscr;", "\355\240\265\355\263\211" },
	{ "&tscy;", "\321\206" },
	{ "&tshcy;", "\321\233" },
	{ "&tstrok;", "\305\247" },
	{ "&twixt;", "\342\211\254" },
	{ "&twoheadleftarrow;", "\342\206\236" },
	{ "&twoheadrightarrow;", "\342\206\240" },
	{ "&uArr;", "\342\207\221" },
	{ "&uHar;", "\342\245\243" },
	{ "&uacute;", "\303\272" },
	{ "&uarr;", "\342\206\221" },
	{ "&ubrcy;", "\321\236" },
	{ "&ubreve;", "\305\255" },
	{ "&ucirc;", "\303\273" },
	{ "&ucy;", "\321\203" },
	{ "&udarr;", "\342\207\205" },
	{ "&udblac;", "\305\261" },
	{ "&udhar;", "\342\245\256" },
	{ "&ufisht;", "\342\245\276" },
	{ "&ufr;", "\355\240\265\355\264\262" },
	{ "&ugrave;", "\303\271" },
	{ "&uharl;", "\342\206\277" },
	{ "&uharr;", "\342\206\276" },
	{ "&uhblk;", "\342\226\200" },
	{ "&ulcorn;", "\342\214\234" },
	{ "&ulcorner;", "\342\214\234" },
	{ "&ulcrop;", "\342\214\217" },
	{ "&ultri;", "\342\227\270" },
	{ "&umacr;", "\305\253" },
	{ "&uml;", "\302\250" },
	{ "&uogon;", "\305\263" },
	{ "&uopf;", "\355\240\265\355\265\246" },
	{ "&uparrow;", "\342\206\221" },
	{ "&updownarrow;", "\342\206\225" },
	{ "&upharpoonleft;", "\342\206\277" },
	{ "&upharpoonright;", "\342\206\276" },
	{ "&uplus;", "\342\212\216" },
	{ "&upsi;", "\317\205" },
	{ "&upsih;", "\317\222" },
	{ "&upsilon;", "\317\205" },
	{ "&upuparrows;", "\342\207\210" },
	{ "&urcorn;", "\342\214\235" },
	{ "&urcorner;", "\342\214\235" },
	{ "&urcrop;", "\342\214\216" },
	{ "&uring;", "\305\257" },
	{ "&urtri;", "\342\227\271" },
	{ "&uscr;", "\355\240\265\355\263\212" },
	{ "&utdot;", "\342\213\260" },
	{ "&utilde;", "\305\251" },
	{ "&utri;", "\342\226\265" },
	{ "&utrif;", "\342\226\264" },
	{ "&uuarr;", "\342\207\210" },
	{ "&uuml;", "\303\274" },
	{ "&uwangle;", "\342\246\247" },
	{ "&vArr;", "\342\207\225" },
	{ "&vBar;", "\342\253\250" },
	{ "&vBarv;", "\342\253\251" },
	{ "&vDash;", "\342\212\250" },
	{ "&vangrt;", "\342\246\234" },
	{ "&varepsilon;", "\317\265" },
	{ "&varkappa;", "\317\260" },
	{ "&varnothing;", "\342\210\205" },
	{ "&varphi;", "\317\225" },
	{ "&varpi;", "\317\226" },
	{ "&varpropto;", "\342\210\235" },
	{ "&varr;", "\342\206\225" },
	{ "&varrho;", "\317\261" },
	{ "&varsigma;", "\317\202" },
	{ "&varsubsetneq;", "\342\212\212\357\270\200" },
	{ "&varsubsetneqq;", "\342\253\213\357\270\200" },
	{ "&varsupsetneq;", "\342\212\213\357\270\200" },
	{ "&varsupsetneqq;", "\342\253\214\357\270\200" },
	{ "&vartheta;", "\317\221" },
	{ "&vartriangleleft;", "\342\212\262" },
	{ "&vartriangleright;", "\342\212\263" },
	{ "&vcy;", "\320\262" },
	{ "&vdash;", "\342\212\242" },
	{ "&vee;", "\342\210\250" },
	{ "&veebar;", "\342\212\273" },
	{ "&veeeq;", "\342\211\232" },
	{ "&vellip;", "\342\213\256" },
	{ "&verbar;", "\174" },	/* | */
	{ "&vert;", "\174" },	/* | */
	{ "&vfr;", "\355\240\265\355\264\263" },
	{ "&vltri;", "\342\212\262" },
	{ "&vnsub;", "\342\212\202\342\203\222" },
	{ "&vnsup;", "\342\212\203\342\203\222" },
	{ "&vopf;", "\355\240\265\355\265\247" },
	{ "&vprop;", "\342\210\235" },
	{ "&vrtri;", "\342\212\263" },
	{ "&vscr;", "\355\240\265\355\263\213" },
	{ "&vsubnE;", "\342\253\213\357\270\200" },
	{ "&vsubne;", "\342\212\212\357\270\200" },
	{ "&vsupnE;", "\342\253\214\357\270\200" },
	{ "&vsupne;", "\342\212\213\357\270\200" },
	{ "&vzigzag;", "\342\246\232" },
	{ "&wcirc;", "\305\265" },
	{ "&wedbar;", "\342\251\237" },
	{ "&wedge;", "\342\210\247" },
	{ "&wedgeq;", "\342\211\231" },
	{ "&weierp;", "\342\204\230" },
	{ "&wfr;", "\355\240\265\355\264\264" },
	{ "&wopf;", "\355\240\265\355\265\250" },
	{ "&wp;", "\342\204\230" },
	{ "&wr;", "\342\211\200" },
	{ "&wreath;", "\342\211\200" },
	{ "&wscr;", "\355\240\265\355\263\214" },
	{ "&xcap;", "\342\213\202" },
	{ "&xcirc;", "\342\227\257" },
	{ "&xcup;", "\342\213\203" },
	{ "&xdtri;", "\342\226\275" },
	{ "&xfr;", "\355\240\265\355\264\265" },
	{ "&xhArr;", "\342\237\272" },
	{ "&xharr;", "\342\237\267" },
	{ "&xi;", "\316\276" },
	{ "&xlArr;", "\342\237\270" },
	{ "&xlarr;", "\342\237\265" },
	{ "&xmap;", "\342\237\274" },
	{ "&xnis;", "\342\213\273" },
	{ "&xodot;", "\342\250\200" },
	{ "&xopf;", "\355\240\265\355\265\251" },
	{ "&xoplus;", "\342\250\201" },
	{ "&xotime;", "\342\250\202" },
	{ "&xrArr;", "\342\237\271" },
	{ "&xrarr;", "\342\237\266" },
	{ "&xscr;", "\355\240\265\355\263\215" },
	{ "&xsqcup;", "\342\250\206" },
	{ "&xuplus;", "\342\250\204" },
	{ "&xutri;", "\342\226\263" },
	{ "&xvee;", "\342\213\201" },
	{ "&xwedge;", "\342\213\200" },
	{ "&yacute;", "\303\275" },
	{ "&yacy;", "\321\217" },
	{ "&ycirc;", "\305\267" },
	{ "&ycy;", "\321\213" },
	{ "&yen;", "\302\245" },
	{ "&yfr;", "\355\240\265\355\264\266" },
	{ "&yicy;", "\321\227" },
	{ "&yopf;", "\355\240\265\355\265\252" },
	{ "&yscr;", "\355\240\265\355\263\216" },
	{ "&yucy;", "\321\216" },
	{ "&yuml;", "\303\277" },
	{ "&zacute;", "\305\272" },
	{ "&zcaron;", "\305\276" },
	{ "&zcy;", "\320\267" },
	{ "&zdot;", "\305\274" },
	{ "&zeetrf;", "\342\204\250" },
	{ "&zeta;", "\316\266" },
	{ "&zfr;", "\355\240\265\355\264\267" },
	{ "&zhcy;", "\320\266" },
	{ "&zigrarr;", "\342\207\235" },
	{ "&zopf;", "\355\240\265\355\265\253" },
	{ "&zscr;", "\355\240\265\355\263\217" },
	{ "&zwj;", "\342\200\215" },
	{ "&zwnj;", "\342\200\214" }
};


/* Function prototypes */
static int	 compare_char_ref(const void *, const void *);
static void	 handle_ampersand(const char *, char *, size_t, size_t *,
    size_t *, bool *);
static inline bool	 is_ascii_whitespace(unsigned long);
static inline bool	 is_control(unsigned long);
static inline bool	 is_non_char(unsigned long);
static inline bool	 is_valid_num_char_ref(unsigned long);


void
text_to_html(const char *t, bool nlx, char *h, size_t *sz)
{
	if (t == NULL) {
		*sz = 0;
		if (h != NULL) {
			*h = '\0';
		}
	} else {
		size_t	 i;
		size_t	 j;
		enum tth_state	 st;

		/*
		 * Output of '\n' to <br> and <p>/</p> conversion is performed
		 * lazily, so that the minimum required markup is emitted.
		 */

		st = TTH_START;

		i = 0;
		j = 0;

		while (t[i] != '\0') {
			if (nlx && t[i] != '\r') {
				switch (st) {

				case TTH_START :
					if (t[i] != '\n') {
						/* Lazy start of paragraph. */
						if (h != NULL) {
							strlcpy(&h[j], "<p>",
							    4);
						}
						j += 3;
						st = TTH_IN_P;
					}
				break;

				case TTH_IN_P :
					if (t[i] == '\n') {
						st = TTH_BR;
					}
				break;

				case TTH_BR :
					if (t[i] == '\n') {
						st = TTH_EOP;
					} else {
						if (h != NULL) {
							/* Lazy linebreak. */
							strlcpy(&h[j],
							    "<br>\n", 6);
						}
						j += 5;
						st = TTH_IN_P;
					}
				break;

				case TTH_EOP :
					if (t[i] != '\n') {
						if (h != NULL) {
							/*
							 * Lazy end/start para.
							 */
							strlcpy(&h[j],
							    "</p>\n<p>", 9);
						}
						j += 8;
						st = TTH_IN_P;
					}
				break;

				}
			}

			switch (t[i]) {

			case '"' :
				if (h != NULL) {
					strlcpy(&h[j], "&quot;", 7);
				}
				j += 6;
			break;

			case '&' :
				if (h != NULL) {
					strlcpy(&h[j], "&amp;", 6);
				}
				j += 5;
			break;

			case '<' :
				if (h != NULL) {
					strlcpy(&h[j], "&lt;", 5);
				}
				j += 4;
			break;

			case '>' :
				if (h != NULL) {
					strlcpy(&h[j], "&gt;", 5);
				}
				j += 4;
			break;

			default :
				if (!nlx || (t[i] != '\n' && t[i] != '\r')) {
					if (h != NULL) {
						h[j] = t[i];
					}
					j += 1;
				}
			break;

			}

			i++;
		}

		if (nlx) {
			/* Complete lazy markup. */
			if (st == TTH_START) {
				if (h != NULL) {
					strlcpy(&h[j], "<p>", 4);
				}
				j += 3;
			}
			if (h != NULL) {
				strlcpy(&h[j], "</p>\n", 6);
			}
			j += 5;
		}

		if (h != NULL) {
			h[j] = '\0';
		}
		j += 1;

		*sz = j;
	}
}


char *
alloc_text_to_html(const char *t, bool nlx)
{
	char	*s;
	size_t	 n;

	s = NULL;

	if (t != NULL) {
		/* Determine required buffer size. */
		text_to_html(t, nlx, NULL, &n);

		if (n > 0 && (s = malloc(n)) != NULL) {
			/* Perform conversion. */
			text_to_html(t, nlx, s, &n);
		}
	}

	return s;
}


/*
 * bsearch() comparison function for string prefix and pointer to
 * struct char_ref_info.
 */
static int
compare_char_ref(const void *a, const void *b)
{
	return strncmp((const char *)a, ((const struct hncr_info *)b)->name,
	    strlen(((const struct hncr_info *)b)->name));
}


/*
 * Returns true if and only if UTF-8 code point cp represents a control
 * character.
 */
static inline bool
is_control(unsigned long cp)
{
	return cp >= 0x0000007F && cp <= 0x0000009F;
}


/*
 * Returns true if and only if UTF-8 code point cp represents ASCII whitespace.
 */
static inline bool
is_ascii_whitespace(unsigned long cp)
{
	return cp == 0x00000009 || cp == 0x0000000A || cp == 0x0000000C
	    || cp == 0x0000000D || cp == 0x00000020;
}


/*
 * Returns true if and only if UTF-8 code point cp represents a non-character.
 */
static inline bool
is_non_char(unsigned long cp)
{
	return cp >= 0x0000FDD0 && cp <= 0x0000FDEF;
}


/*
 * Returns true if and only if r is valid for a numeric character reference.
 */
static inline bool
is_valid_num_char_ref(unsigned long r)
{
	return !(r > UC_CP_MAX || r == 0x0000000D || is_non_char(r)
	    || (is_control(r) && !is_ascii_whitespace(r)));
}


void
html_named_char_ref_utf8_info(const char *h, const char **u, size_t *hsz,
    size_t *usz)
{
	const struct	hncr_info	*info;

	if (h[0] != '&') {
		/* The prefix of h cannot be a named character reference. */
		info = NULL;
	} else {
		info = (const struct hncr_info *)bsearch(h,
		    &html_named_char_refs, sizeof(html_named_char_refs)
		    / sizeof(html_named_char_refs[0]),
		    sizeof(html_named_char_refs[0]), compare_char_ref);
	}

	*u = info != NULL ? info->utf8 : NULL;
	if (hsz != NULL) {
		*hsz = info != NULL ? strlen(info->name) : 0;
	}
	if (usz != NULL) {
		*usz = info != NULL ? strlen(info->utf8) : 0;
	}
}


const char *
html_named_char_ref_to_utf8(const char *h)
{
	const char	*u;

	html_named_char_ref_utf8_info(h, &u, NULL, NULL);

	return u;
}


void
html_num_char_ref_utf8_info(const char *h, uint32_t *cp, size_t *hsz)
{
	/* Initialise to failure settings. */
	*cp = UTF8_INVALID_CP;
	*hsz = 0;

	if (h[0] == '&' && h[1] == '#') {
		unsigned long	 n;
		char	*e;

		/* This may a numeric character reference. */
		errno = 0;
		e = NULL;

		if ((h[2] == 'X' || h[2] == 'x')
		    && isxdigit((unsigned char)h[3])
		    && (h[3] != '0' || h[4] != 'x')) {
			/* Potentially a hexadecimal numeric char reference. */
			n = strtoul(&h[3], &e, 16);
		} else if (isdigit((unsigned char)h[2])) {
			/* Potentially a decimal numeric char reference. */
			n = strtoul(&h[2], &e, 10);
		}

		if (e != NULL && *e == ';') {
			/* We have a number. */
			if ((errno == ERANGE && n == ULONG_MAX)
			    || !is_valid_num_char_ref(n)) {
				/* It is an invalid character reference. */
				n = UTF8_INVALID_CP;
			}
			*cp = (uint32_t)n;
			*hsz = (e - h + 1);
		}
	}
}


/*
 * Helper function for html_to_text().  Deals with input commencing with '&'.
 */
static void
handle_ampersand(const char *h, char *t, size_t sz, size_t *s, size_t *d,
    bool *insuff)
{
	const char	*u;
	size_t	 hsz, usz;
	uint32_t	 cp;
	bool	 handled;

	handled = false;

	if (h[*s + 1] == '#') {
		html_num_char_ref_utf8_info(&h[*s], &cp, &hsz);

		if (hsz > 0) {
			/* It was a numeric character reference. */
			if (cp == 0 || cp == UTF8_INVALID_CP) {
				/* We consider code point U+0000 invalid. */
				if (*d + REPL_LEN < sz) {
					strlcpy(&t[*d], REPL, sz - *d);
					*d += REPL_LEN;
				} else {
					*insuff = true;
				}
			} else {
				char	 buf[5];

				utf8_codept_to_strl(cp, buf, sizeof(buf),
				    &usz);
				if (*d + usz < sz) {
					strlcpy(&t[*d], buf, sz - *d);
					*d += usz;
				} else {
					/* There is insufficient room. */
					*insuff = true;
				}
			}
			*s += hsz;
			handled = true;
		}
	} else {
		html_named_char_ref_utf8_info(&h[*s], &u, &hsz, &usz);
		if (u != NULL) {
			/* It is a named reference. */
			if (*d + usz < sz) {
				strlcpy(&t[*d], u, sz - *d);
				*d += usz;
			} else {
				/* There is insufficient room. */
				*insuff = true;
			}
			*s += hsz;
			handled = true;
		}
	}

	if (!handled) {
		/* Just treat it as a normal character. */
		t[*d] = h[*s];
		*d += 1;
		*s += 1;
	}
}


void
html_to_text(const char *h, bool r, char *t, size_t sz)
{
	size_t	 d, s, z;
	uint32_t	 cp;
	bool	 in_markup, insuff;

	/*
	 * Valid multi-byte encodings are treated atomically.  Thus, there can
	 * be insufficient space in the destination buffer without it being
	 * full.  If that occurs, we cease all appending (other than adding
	 * the terminating NUL).
	 */
	insuff = false;
	in_markup = false;

	s = 0;
	d = 0;

	while (h[s] != '\0' && d + 1 < sz && !insuff) {
		if (isascii((unsigned char)h[s])) {
			if (h[s] == '<') {
				in_markup = true;
				s++;
			} else if (in_markup && h[s] == '>') {
				in_markup = false;
				s++;
			} else if (!in_markup) {
				if (h[s] == '&' && !r) {
					handle_ampersand(h, t, sz, &s, &d,
					    &insuff);
				} else {
					/* Just copy it over. */
					t[d++] = h[s++];
				}
			} else {
				/* In markup, so just skip it. */
				s++;
			}
		} else {
			utf8_strp_to_codept(&h[s], &cp, &z);

			if (d + (cp != UTF8_INVALID_CP ? z : REPL_LEN) >= sz) {
				insuff = true;
			} else {
				strlcpy(&t[d], cp != UTF8_INVALID_CP ? &h[s]
				    : REPL, sz - d);
				d += (cp != UTF8_INVALID_CP ? z : REPL_LEN);
			}

			s += z;
		}
	}

	if (sz > 0) {
		t[d] = '\0';
	}
}


