/* $Id: net_addr.c,v 1.1 2021/04/24 05:05:16 rlr Exp $ */

/*
 * Copyright (c) 2021 Ross Richardson <rueda@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <ctype.h>
#include <stdbool.h>
#include <string.h>

#include "net_addr.h"


/* XD = hexadecimal digits */
#define XD_PER_HEXTET  4
#define HEXTETS_IN_ADDR  8
#define HEXTETS_TO_CMP  4
#define BUF_SZ  (HEXTETS_TO_CMP * XD_PER_HEXTET + 1)


/* Function prototypes. */
static void	 get_hextets(const char *, unsigned int, char *);
static inline bool	 is_all_xdigit_or_colon(const char *);


/* States of parsing FSM */
enum state {
	S_START,
	S_ICOLON,
	S_COLON,
	S_DBLCOLON,
	S_XDIGIT,
	S_FINISH,
	S_NBG
};


/*
 * Copies from a, which must be a valid representation of an IPv6 address,
 * to b, the first nh hextets, inserting elided '0' characters but excluding
 * ':' characters.  Address a must be neither scoped nor in mixed notation.
 * nh must be between 1 and 8 inclusive.  h must be of size sufficient for
 * the requested number of hextets plus a terminating NUL.
 */
static void
get_hextets(const char *a, unsigned int nh, char *h)
{
	enum state	 st;
	size_t	 i, n, o;
	unsigned int	 hextets;

	st = S_START;
	hextets = 0;
	i = 0;
	o = 0;

	while (st != S_FINISH && st != S_NBG) {
		switch(st) {

		case S_START:
			if (a[i] == '\0') {
				st = S_NBG;
			} else if (a[i] == ':') {
				st = S_ICOLON;
			} else {
				n = i;
				st = S_XDIGIT;
			}
			break;

		case S_ICOLON:
			if (a[i] == ':') {
				st = S_DBLCOLON;
			} else {
				st = S_NBG;
			}
			break;

		case S_COLON:
			if (a[i] == '\0') {
				st = S_NBG;
			} else if (a[i] == ':') {
				st = S_DBLCOLON;
			} else {
				n = i;
				st = S_XDIGIT;
			}
			break;

		case S_DBLCOLON:
			if (a[i] == ':') {
				st = S_NBG;
			} else {
				/* Count remaining hextets given. */
				size_t	 m;
				unsigned int	 r;
				bool	 in_xd;

				r = 0;
				in_xd = false;
				m = i;
				while (a[m] != '\0') {
					if (a[m] == ':') {
						in_xd = false;
					} else if (!in_xd) {
						r++;
						in_xd = true;
					}
					m++;
				}
				if (hextets + r > HEXTETS_IN_ADDR - 1) {
					st = S_NBG;
				} else {
					/* Expand the "::". */
					while (hextets < nh
					    && hextets + r < HEXTETS_IN_ADDR) {
						h[o++] = '0'; h[o++] = '0';
						h[o++] = '0'; h[o++] = '0';
						hextets++;
					}
					if (hextets == nh) {
						h[o] = '\0';
						st = S_FINISH;
					} else if (a[i] == '\0') {
						st = S_NBG;
					} else {
						n = i;
						st = S_XDIGIT;
					}
				}
			}
			break;

		case S_XDIGIT:
			if (a[i] == '\0' || a[i] == ':') {
				if (i - n > XD_PER_HEXTET) {
					st = S_NBG;
				} else {
					/* Replace elided leading '0' chars. */
					for (size_t j = 0;
					    j < XD_PER_HEXTET - (i - n); j++) {
						h[o++] = '0';
					}
					while (n < i) {
						h[o++] = tolower(
						    (unsigned char)a[n++]);
					}
					hextets++;
					if (hextets >= nh) {
						h[o] = '\0';
						st = S_FINISH;
					} else if (a[i] == '\0') {
						st = S_NBG;
					} else {
						st = S_COLON;
					}
				}
			}
			break;

		default:
			/* This should not happen. */
			st = S_NBG;
			break;
		};

		i++;
	}

	if (st == S_NBG) {
		h[0] = '\0';
	}
}


/*
 * Returns true iff s (which must be non-NULL) is non-empty and consists
 * only of hexadecimal digits and ':' characters.
 */
static inline bool
is_all_xdigit_or_colon(const char *s)
{
	size_t	 i;

	i = 0;
	while (s[i] != '\0' && (isxdigit((unsigned char)s[i]) || s[i] == ':')) {
		i++;
	}
	return i > 0 && s[i] == '\0';
}


bool
net_addr_match(const char *a, const char *b)
{
	bool	 m;

	m = strcmp(a, b) == 0;

	if (!m && is_all_xdigit_or_colon(a) && is_all_xdigit_or_colon(b)) {
		char	 ax[BUF_SZ];

		/*
		 * Rather than calling inet_pton() or getaddrinfo(), we just
		 * use string manipulation and comparison.
		 */
		get_hextets(a, HEXTETS_TO_CMP, ax);
		if (ax[0] != '\0') {
			char	 bx[BUF_SZ];

			get_hextets(b, HEXTETS_TO_CMP, bx);
			m = strcmp(ax, bx) == 0;
		}
	}

	return m;
}
