/* $Id: utf8_utils.h,v 1.10 2018/06/19 05:02:54 rlr Exp $ */

/*
 * Copyright (c) 2018 Ross L Richardson <rueda@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


/*
 * Module providing rudimentary handling of UTF-8.
 */


#ifndef _UTF8_UTILS_H_
#define _UTF8_UTILS_H_


#include <sys/types.h>

#include <stdbool.h>


/* Unicode is limited to 21 bits. */
#define UTF8_INVALID_CP      0xFFFFFFFFU
/* Replacement used in case of decoding error. */
#define UTF8_REPLACEMENT_CP  0x0000FFFDU


/*
 * bool
 * utf8_is_valid_str(const char *s);
 *
 * Returns true if and only if s is a well-formed UTF-8 string.
 */
bool	 utf8_is_valid_str(const char *);


/*
 * void
 * utf8_codept_to_strl(uint32_t cp, char *dst, size_t dstsz, size_t *encsz);
 *
 * In the absence of error, copies the UTF-8 version of code point cp to dst,
 * which must be of size no less than dstsz (which may be zero).  Unless dstsz
 * is zero, dst is NUL-terminated.  The UTF-8 encoding is treated atomically
 * - that is, either all or none of the bytes are copied to dst.  On
 * completion, *encsz is set to the number of bytes in the encoding (excluding
 * the terminating NUL).  If cp is an invalid code point, *encsz is set to
 * zero.  Note that code point (cp) 0 (NUL) is treated like any other ASCII
 * code point.
 */
void	 utf8_codept_to_strl(uint32_t, char *, size_t, size_t *)
	__attribute__ ((__bounded__ (__string__, 2, 3 )));


/*
 * void
 * utf8_strp_to_codept(const char *s, uint32_t *cp, size_t *encsz);
 *
 * In the absence of error, sets *cp to the code point represented by the
 * initial UTF-8-encoded substring of s, and sets *encsz to the number of
 * bytes in the encoding.  If the prefix of s is the empty string, an invalid
 * encoding, or represents and invalid code point, *cp is set to
 * UTF8_INVALID_CP.  In the absence of error or in the case of a well-formed
 * encoding for an invalid code point, *encsz is set to the full length of the
 * encoding; for the empty string, *encsz is set to zero; for an invalid
 * encoding, *encsz is set to one.
 */
void	 utf8_strp_to_codept(const char *, uint32_t *, size_t *);


/*
 * size_t
 * utf8_strlcpy(char *dst, const char *src, size_t dstsz);
 *
 * Copies (a possibly sanitised version of) UTF-8-encoded string src to dst,
 * which must be of length no less than dstsz.  UTF-8-encoded code points are
 * treated atomically - that is, they are copied completely or not at all.
 * Invalid encodings in src are replaced by the encoding for code point
 * UTF8_REPLACEMENT_CP.  No more than dstsize - 1 bytes plus a terminating NUL
 * are copied to dst.  Unless dstsz is zero, dst is NUL-terminated.  Returns
 * the total length (in bytes) which the dst string would occupy (excluding the
 * terminating NUL) given sufficient space.
 */
size_t	 utf8_strlcpy(char *, const char *, size_t)
	__attribute__ ((__bounded__ (__string__, 1, 3 )));


#endif /* !_UTF8_UTILS_H_ */

